C @(#)fttokn.for	19.1 (ES0-DMD) 02/25/03 13:17:37
C===========================================================================
C Copyright (C) 1995 European Southern Observatory (ESO)
C
C This program is free software; you can redistribute it and/or 
C modify it under the terms of the GNU General Public License as 
C published by the Free Software Foundation; either version 2 of 
C the License, or (at your option) any later version.
C
C This program is distributed in the hope that it will be useful,
C but WITHOUT ANY WARRANTY; without even the implied warranty of
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C GNU General Public License for more details.
C
C You should have received a copy of the GNU General Public 
C License along with this program; if not, write to the Free 
C Software Foundation, Inc., 675 Massachusetss Ave, Cambridge, 
C MA 02139, USA.
C
C Corresponding concerning ESO-MIDAS should be addressed as follows:
C	Internet e-mail: midas@eso.org
C	Postal address: European Southern Observatory
C			Data Management Division 
C			Karl-Schwarzschild-Strasse 2
C			D 85748 Garching bei Muenchen 
C			GERMANY
C===========================================================================
C
      SUBROUTINE FTTOKN(LINE,EOFMRK,DELIM,TOKEN,LENGTH,EOF)                    
C++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++                   
C.COPYRIGHT: Copyright (c) 1987 European Southern Observatory,                  
C                                         all rights reserved                   
C                                                                               
C.VERSION: 1.0  ESO-FORTRAN Conversion, AA  17:25 - 13 JAN 1988                 
C                                                                               
C.LANGUAGE: F77+ESOext                                                          
C                                                                               
C.AUTHOR: J.D.PONZ                                                              
C                                                                               
C.IDENTIFICATION                                                                
C                                                                               
C  FITLIB.FOR   VERSION 1.0  27 MAR 1984                                        
C                                                                               
C.PURPOSE                                                                       
C                                                                               
C  INTERFACE ROUTINES FOR THE FITTING STRUCTURES                                
C                                                                               
C.ALGORITHM                                                                     
C                                                                               
C  USE MIDAS I/O INTERFACES TO FRAMES AND TABLES                                
C                                                                               
C.KEYWORDS                                                                      
C                                                                               
C  NON LINEAR FITTING                                                           
C                                                                               
C                                                                               
C----------------------------------------------------------------               
C                                                                               
C FIND NEXT TOKEN                                                               
C      IMPLICIT NONE                                                            
C                                                                               
      INTEGER I,LENGTH
      CHARACTER*(*) LINE,TOKEN                                                  
      LOGICAL EOF
      CHARACTER*1 EOFMRK,DELIM,C                                               
C                                                                               
      EOF    = .FALSE.                                                          
      C      = ' '                                                              
      I      = 0                                                                
99    CONTINUE
          IF ( .NOT. (C.EQ.' ')) GO TO 20                                       
          I      = I + 1                                                        
          C      = LINE(I:I)                                                    
      GOTO 99
   20 CONTINUE                                                                  
      I      = I - 1                                                            
      LENGTH = 0                                                                
      TOKEN  = ' '                                                              
999   CONTINUE
          IF ( .NOT. (.NOT.EOF)) GO TO 40                                       
          I      = I + 1                                                        
          C      = LINE(I:I)                                                    
          EOF    = C .EQ. EOFMRK                                               
          IF (C.EQ.DELIM) THEN                                                  
              RETURN                                                            
                                                                                
          ELSE                                                                  
              LENGTH = LENGTH + 1                                               
              TOKEN(LENGTH:LENGTH) = C                                          
          END IF                                                                
                                                                                
      GOTO 999
   40 CONTINUE                                                                  
      IF (EOF) THEN                                                             
          LENGTH = LENGTH - 1                                                   
          IF (LENGTH.NE.0) EOF    = .FALSE.                                     
      END IF                                                                    
C 
      RETURN                                                                    
      END                                                                       
