.\" @(#)gomidas.l	19.1 (ESO-IPG) 02/25/03 14:32:14
.\" Copyright (c) 1989 European Southern Observatory.
.\" All rights reserved.  
.\"
.TH gomidas 1L "07 July 1993" MIDAS
.SH NAME
gomidas \- Resume a MIDAS session.
.SH SYNOPSIS
.B gomidas
[
.B \-d
.I display
] [ 
.B \-m
.I mid_work
]
.SH DESCRIPTION
.I gomidas 
resumes a previous exited MIDAS session which was saved
by 
.I inmidas
(just before exiting) in a file located in the 
.IR MID_WORK 
directory.
.PP
If 
.I inmidas
was executed in PARALLEL mode there could be several 
files containing different MIDAS sessions saved in 
.IR MIDWORK.
In this case, 
.I gomidas
will read the one associated to its terminal if that file exists, otherwise 
it will read the one associated to the latest MIDAS session that exited.

.SH OPTIONS
.IP "\fB-m\fp \fImid_work\fp"
Startup directory for MIDAS containing previous MIDAS environment files.
.IP "\fB-d\fp \fIdisplay\fp"
Specifies another X server for the display and graphical MIDAS windows (
.B NOTE:
be aware of allowed access to a remote X server using the
.IR xhost (1) 
command.)
.SH ENVIRONMENT
.I inmidas
can also use the following definitions in 
the environment with the same performance as previous described arguments.
.br
.B NOTE:
Arguments in the 
.I inmidas
command line are taken with preference to environment variables.
.IP "\fBDISPLAY\fp"
X server for the display and graphical MIDAS windows.
.IP "\fBMID_WORK\fp"
Startup directory for MIDAS containing previous MIDAS saved-session files.
.SH SEE ALSO
.IR inmidas(1L)
