/* @(#)UxPutsMF.h	17.1 (ESO-IPG) 01/25/02 17:26:01 */
#ifndef	_UXPUTSMF_INCLUDED
#define	_UXPUTSMF_INCLUDED
#define	UxPutWmTimeout(o, v)	UxPutProp(o, XmNwmTimeout, ((XtArgVal)(v)))
#define	UxPutWinGravity(o, v)	UxDDPutProp(o, XmNwinGravity, ((XtArgVal)(v)))
#define	UxPutWindowGroup(o, v)	UxDDPutProp(o, XmNwindowGroup, ((XtArgVal)(v)))
#define	UxPutWidthInc(o, v)	UxPutProp(o, XmNwidthInc, ((XtArgVal)(v)))
#define	UxPutWaitForWm(o, v)	UxDDPutProp(o, XmNwaitForWm, ((XtArgVal)(v)))
#define	UxPutTransient(o, v)	UxDDPutProp(o, XmNtransient, ((XtArgVal)(v)))
#define	UxPutTitleEncoding(o, v)	UxDDPutProp(o, XmNtitleEncoding, ((XtArgVal)(v)))
#define	UxPutTitle(o, v)	UxPutProp(o, XmNtitle, ((XtArgVal)(v)))
#define	UxPutMinWidth(o, v)	UxPutProp(o, XmNminWidth, ((XtArgVal)(v)))
#define	UxPutMinHeight(o, v)	UxPutProp(o, XmNminHeight, ((XtArgVal)(v)))
#define	UxPutMinAspectY(o, v)	UxPutProp(o, XmNminAspectY, ((XtArgVal)(v)))
#define	UxPutMinAspectX(o, v)	UxPutProp(o, XmNminAspectX, ((XtArgVal)(v)))
#define	UxPutMaxWidth(o, v)	UxPutProp(o, XmNmaxWidth, ((XtArgVal)(v)))
#define	UxPutMaxHeight(o, v)	UxPutProp(o, XmNmaxHeight, ((XtArgVal)(v)))
#define	UxPutMaxAspectY(o, v)	UxPutProp(o, XmNmaxAspectY, ((XtArgVal)(v)))
#define	UxPutMaxAspectX(o, v)	UxPutProp(o, XmNmaxAspectX, ((XtArgVal)(v)))
#define	UxPutInput(o, v)	UxDDPutProp(o, XmNinput, ((XtArgVal)(v)))
#define	UxPutInitialState(o, v)	UxDDPutProp(o, XmNinitialState, ((XtArgVal)(v)))
#define	UxPutIconY(o, v)	UxPutProp(o, XmNiconY, ((XtArgVal)(v)))
#define	UxPutIconX(o, v)	UxPutProp(o, XmNiconX, ((XtArgVal)(v)))
#define	UxPutIconWindow(o, v)	UxDDPutProp(o, XmNiconWindow, ((XtArgVal)(v)))
#define	UxPutIconPixmap(o, v)	UxDDPutProp(o, XmNiconPixmap, ((XtArgVal)(v)))
#define	UxPutIconMask(o, v)	UxDDPutProp(o, XmNiconMask, ((XtArgVal)(v)))
#define	UxPutHeightInc(o, v)	UxPutProp(o, XmNheightInc, ((XtArgVal)(v)))
#define	UxPutBaseWidth(o, v)	UxPutProp(o, XmNbaseWidth, ((XtArgVal)(v)))
#define	UxPutBaseHeight(o, v)	UxPutProp(o, XmNbaseHeight, ((XtArgVal)(v)))
#define	UxPutUseAsyncGeometry(o, v)	UxDDPutProp(o, XmNuseAsyncGeometry, ((XtArgVal)(v)))
#define	UxPutShellUnitType(o, v)	UxDDPutProp(o, XmNshellUnitType, ((XtArgVal)(v)))
#define	UxPutPreeditType(o, v)	UxPutProp(o, XmNpreeditType, ((XtArgVal)(v)))
#define	UxPutMwmMenu(o, v)	UxPutProp(o, XmNmwmMenu, ((XtArgVal)(v)))
#define	UxPutMwmInputMode(o, v)	UxDDPutProp(o, XmNmwmInputMode, ((XtArgVal)(v)))
#define	UxPutMwmFunctions(o, v)	UxPutProp(o, XmNmwmFunctions, ((XtArgVal)(v)))
#define	UxPutMwmDecorations(o, v)	UxPutProp(o, XmNmwmDecorations, ((XtArgVal)(v)))
#define	UxPutKeyboardFocusPolicy(o, v)	UxDDPutProp(o, XmNkeyboardFocusPolicy, ((XtArgVal)(v)))
#define	UxPutInputMethod(o, v)	UxPutProp(o, XmNinputMethod, ((XtArgVal)(v)))
#define	UxPutDeleteResponse(o, v)	UxDDPutProp(o, XmNdeleteResponse, ((XtArgVal)(v)))
#define	UxPutAudibleWarning(o, v)	UxDDPutProp(o, XmNaudibleWarning, ((XtArgVal)(v)))
#define	UxPutTransientFor(o, v)	UxDDPutProp(o, XmNtransientFor, ((XtArgVal)(v)))
#define	UxPutIconNameEncoding(o, v)	UxDDPutProp(o, XmNiconNameEncoding, ((XtArgVal)(v)))
#define	UxPutIconName(o, v)	UxPutProp(o, XmNiconName, ((XtArgVal)(v)))
#define	UxPutIconic(o, v)	UxDDPutProp(o, XmNiconic, ((XtArgVal)(v)))
#define	UxPutVisibleWhenOff(o, v)	UxDDPutProp(o, XmNvisibleWhenOff, ((XtArgVal)(v)))
#define	UxPutSet(o, v)	UxDDPutProp(o, XmNset, ((XtArgVal)(v)))
#define	UxPutSelectPixmap(o, v)	UxDDPutProp(o, XmNselectPixmap, ((XtArgVal)(v)))
#define	UxPutSelectInsensitivePixmap(o, v)	UxDDPutProp(o, XmNselectInsensitivePixmap, ((XtArgVal)(v)))
#define	UxPutSelectColor(o, v)	UxDDPutProp(o, XmNselectColor, ((XtArgVal)(v)))
#define	UxPutIndicatorType(o, v)	UxDDPutProp(o, XmNindicatorType, ((XtArgVal)(v)))
#define	UxPutIndicatorSize(o, v)	UxDDPutProp(o, XmNindicatorSize, ((XtArgVal)(v)))
#define	UxPutIndicatorOn(o, v)	UxDDPutProp(o, XmNindicatorOn, ((XtArgVal)(v)))
#define	UxPutFillOnSelect(o, v)	UxDDPutProp(o, XmNfillOnSelect, ((XtArgVal)(v)))
#define	UxPutWordWrap(o, v)	UxDDPutProp(o, XmNwordWrap, ((XtArgVal)(v)))
#define	UxPutRows(o, v)	UxPutProp(o, XmNrows, ((XtArgVal)(v)))
#define	UxPutCursorPositionVisible(o, v)	UxDDPutProp(o, XmNcursorPositionVisible, ((XtArgVal)(v)))
#define	UxPutColumns(o, v)	UxPutProp(o, XmNcolumns, ((XtArgVal)(v)))
#define	UxPutBlinkRate(o, v)	UxPutProp(o, XmNblinkRate, ((XtArgVal)(v)))
#define	UxPutSource(o, v)	UxPutProp(o, XmNsource, ((XtArgVal)(v)))
#define	UxPutSelectThreshold(o, v)	UxPutProp(o, XmNselectThreshold, ((XtArgVal)(v)))
#define	UxPutSelectionArrayCount(o, v)	UxPutProp(o, XmNselectionArrayCount, ((XtArgVal)(v)))
#define	UxPutSelectionArray(o, v)	UxDDPutProp(o, XmNselectionArray, ((XtArgVal)(v)))
#define	UxPutPendingDelete(o, v)	UxDDPutProp(o, XmNpendingDelete, ((XtArgVal)(v)))
#define	UxPutVerifyBell(o, v)	UxDDPutProp(o, XmNverifyBell, ((XtArgVal)(v)))
#define	UxPutValueWcs(o, v)	UxDDPutProp(o, XmNvalueWcs, ((XtArgVal)(v)))
#define	UxPutTopCharacter(o, v)	UxPutProp(o, XmNtopCharacter, ((XtArgVal)(v)))
#define	UxPutMaxLength(o, v)	UxPutProp(o, XmNmaxLength, ((XtArgVal)(v)))
#define	UxPutEditMode(o, v)	UxDDPutProp(o, XmNeditMode, ((XtArgVal)(v)))
#define	UxPutEditable(o, v)	UxDDPutProp(o, XmNeditable, ((XtArgVal)(v)))
#define	UxPutCursorPosition(o, v)	UxPutProp(o, XmNcursorPosition, ((XtArgVal)(v)))
#define	UxPutAutoShowCursorPosition(o, v)	UxDDPutProp(o, XmNautoShowCursorPosition, ((XtArgVal)(v)))
#define	UxPutVisual(o, v)	UxPutProp(o, XmNvisual, ((XtArgVal)(v)))
#define	UxPutSaveUnder(o, v)	UxDDPutProp(o, XmNsaveUnder, ((XtArgVal)(v)))
#define	UxPutGeometry(o, v)	UxDDPutProp(o, XmNgeometry, ((XtArgVal)(v)))
#define	UxPutCreatePopupChildProc(o, v)	UxPutProp(o, XmNcreatePopupChildProc, ((XtArgVal)(v)))
#define	UxPutSeparatorType(o, v)	UxDDPutProp(o, XmNseparatorType, ((XtArgVal)(v)))
#define	UxPutMargin(o, v)	UxDDPutProp(o, XmNmargin, ((XtArgVal)(v)))
#define	UxPutTextString(o, v)	UxDDPutProp(o, XmNtextString, ((XtArgVal)(v)))
#define	UxPutTextColumns(o, v)	UxPutProp(o, XmNtextColumns, ((XtArgVal)(v)))
#define	UxPutTextAccelerators(o, v)	UxDDPutProp(o, XmNtextAccelerators, ((XtArgVal)(v)))
#define	UxPutSelectionLabelString(o, v)	UxDDPutProp(o, XmNselectionLabelString, ((XtArgVal)(v)))
#define	UxPutMustMatch(o, v)	UxDDPutProp(o, XmNmustMatch, ((XtArgVal)(v)))
#define	UxPutListVisibleItemCount(o, v)	UxPutProp(o, XmNlistVisibleItemCount, ((XtArgVal)(v)))
#define	UxPutListLabelString(o, v)	UxDDPutProp(o, XmNlistLabelString, ((XtArgVal)(v)))
#define	UxPutListItemCount(o, v)	UxPutProp(o, XmNlistItemCount, ((XtArgVal)(v)))
#define	UxPutListItems(o, v)	UxDDPutProp(o, XmNlistItems, ((XtArgVal)(v)))
#define	UxPutDialogType(o, v)	UxDDPutProp(o, XmNdialogType, ((XtArgVal)(v)))
#define	UxPutChildPlacement(o, v)	UxDDPutProp(o, XmNchildPlacement, ((XtArgVal)(v)))
#define	UxPutApplyLabelString(o, v)	UxDDPutProp(o, XmNapplyLabelString, ((XtArgVal)(v)))
#define	UxPutVisualPolicy(o, v)	UxDDPutProp(o, XmNvisualPolicy, ((XtArgVal)(v)))
#define	UxPutScrollingPolicy(o, v)	UxDDPutProp(o, XmNscrollingPolicy, ((XtArgVal)(v)))
#define	UxPutScrolledWindowMarginWidth(o, v)	UxDDPutProp(o, XmNscrolledWindowMarginWidth, ((XtArgVal)(v)))
#define	UxPutScrolledWindowMarginHeight(o, v)	UxDDPutProp(o, XmNscrolledWindowMarginHeight, ((XtArgVal)(v)))
#define	UxPutScrollBarPlacement(o, v)	UxDDPutProp(o, XmNscrollBarPlacement, ((XtArgVal)(v)))
#define	UxPutTroughColor(o, v)	UxDDPutProp(o, XmNtroughColor, ((XtArgVal)(v)))
#define	UxPutSliderSize(o, v)	UxPutProp(o, XmNsliderSize, ((XtArgVal)(v)))
#define	UxPutShowArrows(o, v)	UxDDPutProp(o, XmNshowArrows, ((XtArgVal)(v)))
#define	UxPutRepeatDelay(o, v)	UxPutProp(o, XmNrepeatDelay, ((XtArgVal)(v)))
#define	UxPutPageIncrement(o, v)	UxPutProp(o, XmNpageIncrement, ((XtArgVal)(v)))
#define	UxPutInitialDelay(o, v)	UxPutProp(o, XmNinitialDelay, ((XtArgVal)(v)))
#define	UxPutIncrement(o, v)	UxPutProp(o, XmNincrement, ((XtArgVal)(v)))
#define	UxPutValue(o, v)	UxPutProp(o, XmNvalue, ((XtArgVal)(v)))
#define	UxPutTitleString(o, v)	UxDDPutProp(o, XmNtitleString, ((XtArgVal)(v)))
#define	UxPutShowValue(o, v)	UxDDPutProp(o, XmNshowValue, ((XtArgVal)(v)))
#define	UxPutScaleWidth(o, v)	UxDDPutProp(o, XmNscaleWidth, ((XtArgVal)(v)))
#define	UxPutScaleMultiple(o, v)	UxPutProp(o, XmNscaleMultiple, ((XtArgVal)(v)))
#define	UxPutScaleHeight(o, v)	UxDDPutProp(o, XmNscaleHeight, ((XtArgVal)(v)))
#define	UxPutProcessingDirection(o, v)	UxDDPutProp(o, XmNprocessingDirection, ((XtArgVal)(v)))
#define	UxPutMinimum(o, v)	UxPutProp(o, XmNminimum, ((XtArgVal)(v)))
#define	UxPutMaximum(o, v)	UxPutProp(o, XmNmaximum, ((XtArgVal)(v)))
#define	UxPutDecimalPoints(o, v)	UxPutProp(o, XmNdecimalPoints, ((XtArgVal)(v)))
#define	UxPutScrollVertical(o, v)	UxDDPutProp(o, XmNscrollVertical, ((XtArgVal)(v)))
#define	UxPutScrollTopSide(o, v)	UxDDPutProp(o, XmNscrollTopSide, ((XtArgVal)(v)))
#define	UxPutScrollLeftSide(o, v)	UxDDPutProp(o, XmNscrollLeftSide, ((XtArgVal)(v)))
#define	UxPutScrollHorizontal(o, v)	UxDDPutProp(o, XmNscrollHorizontal, ((XtArgVal)(v)))
#define	UxPutWhichButton(o, v)	UxPutProp(o, XmNwhichButton, ((XtArgVal)(v)))
#define	UxPutTearOffModel(o, v)	UxDDPutProp(o, XmNtearOffModel, ((XtArgVal)(v)))
#define	UxPutRowColumnType(o, v)	UxDDPutProp(o, XmNrowColumnType, ((XtArgVal)(v)))
#define	UxPutResizeWidth(o, v)	UxDDPutProp(o, XmNresizeWidth, ((XtArgVal)(v)))
#define	UxPutResizeHeight(o, v)	UxDDPutProp(o, XmNresizeHeight, ((XtArgVal)(v)))
#define	UxPutRadioBehavior(o, v)	UxDDPutProp(o, XmNradioBehavior, ((XtArgVal)(v)))
#define	UxPutRadioAlwaysOne(o, v)	UxDDPutProp(o, XmNradioAlwaysOne, ((XtArgVal)(v)))
#define	UxPutPopupEnabled(o, v)	UxDDPutProp(o, XmNpopupEnabled, ((XtArgVal)(v)))
#define	UxPutPacking(o, v)	UxDDPutProp(o, XmNpacking, ((XtArgVal)(v)))
#define	UxPutOrientation(o, v)	UxDDPutProp(o, XmNorientation, ((XtArgVal)(v)))
#define	UxPutNumColumns(o, v)	UxPutProp(o, XmNnumColumns, ((XtArgVal)(v)))
#define	UxPutMenuPost(o, v)	UxDDPutProp(o, XmNmenuPost, ((XtArgVal)(v)))
#define	UxPutMenuHistory(o, v)	UxDDPutProp(o, XmNmenuHistory, ((XtArgVal)(v)))
#define	UxPutMenuHelpWidget(o, v)	UxDDPutProp(o, XmNmenuHelpWidget, ((XtArgVal)(v)))
#define	UxPutMenuAccelerator(o, v)	UxPutProp(o, XmNmenuAccelerator, ((XtArgVal)(v)))
#define	UxPutIsHomogeneous(o, v)	UxDDPutProp(o, XmNisHomogeneous, ((XtArgVal)(v)))
#define	UxPutIsAligned(o, v)	UxDDPutProp(o, XmNisAligned, ((XtArgVal)(v)))
#define	UxPutEntryVerticalAlignment(o, v)	UxDDPutProp(o, XmNentryVerticalAlignment, ((XtArgVal)(v)))
#define	UxPutEntryClass(o, v)	UxDDPutProp(o, XmNentryClass, ((XtArgVal)(v)))
#define	UxPutEntryBorder(o, v)	UxDDPutProp(o, XmNentryBorder, ((XtArgVal)(v)))
#define	UxPutEntryAlignment(o, v)	UxDDPutProp(o, XmNentryAlignment, ((XtArgVal)(v)))
#define	UxPutAdjustMargin(o, v)	UxDDPutProp(o, XmNadjustMargin, ((XtArgVal)(v)))
#define	UxPutAdjustLast(o, v)	UxDDPutProp(o, XmNadjustLast, ((XtArgVal)(v)))
#define	UxPutShowAsDefault(o, v)	UxDDPutProp(o, XmNshowAsDefault, ((XtArgVal)(v)))
#define	UxPutFillOnArm(o, v)	UxDDPutProp(o, XmNfillOnArm, ((XtArgVal)(v)))
#define	UxPutDefaultButtonShadowThickness(o, v)	UxDDPutProp(o, XmNdefaultButtonShadowThickness, ((XtArgVal)(v)))
#define	UxPutArmPixmap(o, v)	UxDDPutProp(o, XmNarmPixmap, ((XtArgVal)(v)))
#define	UxPutArmColor(o, v)	UxDDPutProp(o, XmNarmColor, ((XtArgVal)(v)))
#define	UxPutSkipAdjust(o, v)	UxDDPutProp(o, XmNskipAdjust, ((XtArgVal)(v)))
#define	UxPutPositionIndex(o, v)	UxPutProp(o, XmNpositionIndex, ((XtArgVal)(v)))
#define	UxPutPaneMinimum(o, v)	UxDDPutProp(o, XmNpaneMinimum, ((XtArgVal)(v)))
#define	UxPutPaneMaximum(o, v)	UxDDPutProp(o, XmNpaneMaximum, ((XtArgVal)(v)))
#define	UxPutAllowResize(o, v)	UxDDPutProp(o, XmNallowResize, ((XtArgVal)(v)))
#define	UxPutSpacing(o, v)	UxDDPutProp(o, XmNspacing, ((XtArgVal)(v)))
#define	UxPutSeparatorOn(o, v)	UxDDPutProp(o, XmNseparatorOn, ((XtArgVal)(v)))
#define	UxPutSashWidth(o, v)	UxDDPutProp(o, XmNsashWidth, ((XtArgVal)(v)))
#define	UxPutSashShadowThickness(o, v)	UxDDPutProp(o, XmNsashShadowThickness, ((XtArgVal)(v)))
#define	UxPutSashIndent(o, v)	UxDDPutProp(o, XmNsashIndent, ((XtArgVal)(v)))
#define	UxPutSashHeight(o, v)	UxDDPutProp(o, XmNsashHeight, ((XtArgVal)(v)))
#define	UxPutRefigureMode(o, v)	UxDDPutProp(o, XmNrefigureMode, ((XtArgVal)(v)))
#define	UxPutSymbolPixmap(o, v)	UxDDPutProp(o, XmNsymbolPixmap, ((XtArgVal)(v)))
#define	UxPutOkLabelString(o, v)	UxDDPutProp(o, XmNokLabelString, ((XtArgVal)(v)))
#define	UxPutMinimizeButtons(o, v)	UxDDPutProp(o, XmNminimizeButtons, ((XtArgVal)(v)))
#define	UxPutMessageString(o, v)	UxDDPutProp(o, XmNmessageString, ((XtArgVal)(v)))
#define	UxPutMessageAlignment(o, v)	UxDDPutProp(o, XmNmessageAlignment, ((XtArgVal)(v)))
#define	UxPutHelpLabelString(o, v)	UxDDPutProp(o, XmNhelpLabelString, ((XtArgVal)(v)))
#define	UxPutDefaultButtonType(o, v)	UxDDPutProp(o, XmNdefaultButtonType, ((XtArgVal)(v)))
#define	UxPutCancelLabelString(o, v)	UxDDPutProp(o, XmNcancelLabelString, ((XtArgVal)(v)))
#define	UxPutTopShadowPixmap(o, v)	UxDDPutProp(o, XmNtopShadowPixmap, ((XtArgVal)(v)))
#define	UxPutTopShadowColor(o, v)	UxDDPutProp(o, XmNtopShadowColor, ((XtArgVal)(v)))
#define	UxPutInitialFocus(o, v)	UxDDPutProp(o, XmNinitialFocus, ((XtArgVal)(v)))
#define	UxPutHighlightPixmap(o, v)	UxDDPutProp(o, XmNhighlightPixmap, ((XtArgVal)(v)))
#define	UxPutHighlightColor(o, v)	UxDDPutProp(o, XmNhighlightColor, ((XtArgVal)(v)))
#define	UxPutForeground(o, v)	UxDDPutProp(o, XmNforeground, ((XtArgVal)(v)))
#define	UxPutBottomShadowPixmap(o, v)	UxDDPutProp(o, XmNbottomShadowPixmap, ((XtArgVal)(v)))
#define	UxPutBottomShadowColor(o, v)	UxDDPutProp(o, XmNbottomShadowColor, ((XtArgVal)(v)))
#define	UxPutAllowShellResize(o, v)	UxDDPutProp(o, XmNallowShellResize, ((XtArgVal)(v)))
#define	UxPutDefaultFontList(o, v)	UxDDPutProp(o, XmNdefaultFontList, ((XtArgVal)(v)))
#define	UxPutShowSeparator(o, v)	UxDDPutProp(o, XmNshowSeparator, ((XtArgVal)(v)))
#define	UxPutMainWindowMarginWidth(o, v)	UxDDPutProp(o, XmNmainWindowMarginWidth, ((XtArgVal)(v)))
#define	UxPutMainWindowMarginHeight(o, v)	UxDDPutProp(o, XmNmainWindowMarginHeight, ((XtArgVal)(v)))
#define	UxPutCommandWindowLocation(o, v)	UxDDPutProp(o, XmNcommandWindowLocation, ((XtArgVal)(v)))
#define	UxPutVisibleItemCount(o, v)	UxPutProp(o, XmNvisibleItemCount, ((XtArgVal)(v)))
#define	UxPutTopItemPosition(o, v)	UxPutProp(o, XmNtopItemPosition, ((XtArgVal)(v)))
#define	UxPutSelectionPolicy(o, v)	UxDDPutProp(o, XmNselectionPolicy, ((XtArgVal)(v)))
#define	UxPutSelectedItemCount(o, v)	UxPutProp(o, XmNselectedItemCount, ((XtArgVal)(v)))
#define	UxPutSelectedItems(o, v)	UxDDPutProp(o, XmNselectedItems, ((XtArgVal)(v)))
#define	UxPutScrollBarDisplayPolicy(o, v)	UxDDPutProp(o, XmNscrollBarDisplayPolicy, ((XtArgVal)(v)))
#define	UxPutListSpacing(o, v)	UxDDPutProp(o, XmNlistSpacing, ((XtArgVal)(v)))
#define	UxPutListSizePolicy(o, v)	UxDDPutProp(o, XmNlistSizePolicy, ((XtArgVal)(v)))
#define	UxPutListMarginWidth(o, v)	UxDDPutProp(o, XmNlistMarginWidth, ((XtArgVal)(v)))
#define	UxPutListMarginHeight(o, v)	UxDDPutProp(o, XmNlistMarginHeight, ((XtArgVal)(v)))
#define	UxPutItemCount(o, v)	UxPutProp(o, XmNitemCount, ((XtArgVal)(v)))
#define	UxPutItems(o, v)	UxDDPutProp(o, XmNitems, ((XtArgVal)(v)))
#define	UxPutDoubleClickInterval(o, v)	UxPutProp(o, XmNdoubleClickInterval, ((XtArgVal)(v)))
#define	UxPutAutomaticSelection(o, v)	UxDDPutProp(o, XmNautomaticSelection, ((XtArgVal)(v)))
#define	UxPutStringDirection(o, v)	UxDDPutProp(o, XmNstringDirection, ((XtArgVal)(v)))
#define	UxPutRecomputeSize(o, v)	UxDDPutProp(o, XmNrecomputeSize, ((XtArgVal)(v)))
#define	UxPutMnemonicCharSet(o, v)	UxPutProp(o, XmNmnemonicCharSet, ((XtArgVal)(v)))
#define	UxPutMnemonic(o, v)	UxDDPutProp(o, XmNmnemonic, ((XtArgVal)(v)))
#define	UxPutMarginTop(o, v)	UxDDPutProp(o, XmNmarginTop, ((XtArgVal)(v)))
#define	UxPutMarginRight(o, v)	UxDDPutProp(o, XmNmarginRight, ((XtArgVal)(v)))
#define	UxPutMarginLeft(o, v)	UxDDPutProp(o, XmNmarginLeft, ((XtArgVal)(v)))
#define	UxPutMarginBottom(o, v)	UxDDPutProp(o, XmNmarginBottom, ((XtArgVal)(v)))
#define	UxPutLabelType(o, v)	UxDDPutProp(o, XmNlabelType, ((XtArgVal)(v)))
#define	UxPutLabelString(o, v)	UxDDPutProp(o, XmNlabelString, ((XtArgVal)(v)))
#define	UxPutLabelPixmap(o, v)	UxDDPutProp(o, XmNlabelPixmap, ((XtArgVal)(v)))
#define	UxPutLabelInsensitivePixmap(o, v)	UxDDPutProp(o, XmNlabelInsensitivePixmap, ((XtArgVal)(v)))
#define	UxPutFontList(o, v)	UxDDPutProp(o, XmNfontList, ((XtArgVal)(v)))
#define	UxPutAlignment(o, v)	UxDDPutProp(o, XmNalignment, ((XtArgVal)(v)))
#define	UxPutAcceleratorText(o, v)	UxDDPutProp(o, XmNacceleratorText, ((XtArgVal)(v)))
#define	UxPutAccelerator(o, v)	UxPutProp(o, XmNaccelerator, ((XtArgVal)(v)))
#define	UxPutUserData(o, v)	UxDDPutProp(o, XmNuserData, ((XtArgVal)(v)))
#define	UxPutUnitType(o, v)	UxDDPutProp(o, XmNunitType, ((XtArgVal)(v)))
#define	UxPutTraversalOn(o, v)	UxDDPutProp(o, XmNtraversalOn, ((XtArgVal)(v)))
#define	UxPutShadowThickness(o, v)	UxDDPutProp(o, XmNshadowThickness, ((XtArgVal)(v)))
#define	UxPutNavigationType(o, v)	UxDDPutProp(o, XmNnavigationType, ((XtArgVal)(v)))
#define	UxPutHighlightThickness(o, v)	UxDDPutProp(o, XmNhighlightThickness, ((XtArgVal)(v)))
#define	UxPutHighlightOnEnter(o, v)	UxDDPutProp(o, XmNhighlightOnEnter, ((XtArgVal)(v)))
#define	UxPutQualifySearchDataProc(o, v)	UxPutProp(o, XmNqualifySearchDataProc, ((XtArgVal)(v)))
#define	UxPutPattern(o, v)	UxDDPutProp(o, XmNpattern, ((XtArgVal)(v)))
#define	UxPutNoMatchString(o, v)	UxDDPutProp(o, XmNnoMatchString, ((XtArgVal)(v)))
#define	UxPutListUpdated(o, v)	UxDDPutProp(o, XmNlistUpdated, ((XtArgVal)(v)))
#define	UxPutFilterLabelString(o, v)	UxDDPutProp(o, XmNfilterLabelString, ((XtArgVal)(v)))
#define	UxPutFileTypeMask(o, v)	UxDDPutProp(o, XmNfileTypeMask, ((XtArgVal)(v)))
#define	UxPutFileSearchProc(o, v)	UxPutProp(o, XmNfileSearchProc, ((XtArgVal)(v)))
#define	UxPutFileListLabelString(o, v)	UxDDPutProp(o, XmNfileListLabelString, ((XtArgVal)(v)))
#define	UxPutFileListItemCount(o, v)	UxPutProp(o, XmNfileListItemCount, ((XtArgVal)(v)))
#define	UxPutFileListItems(o, v)	UxDDPutProp(o, XmNfileListItems, ((XtArgVal)(v)))
#define	UxPutDirSpec(o, v)	UxDDPutProp(o, XmNdirSpec, ((XtArgVal)(v)))
#define	UxPutDirSearchProc(o, v)	UxPutProp(o, XmNdirSearchProc, ((XtArgVal)(v)))
#define	UxPutDirMask(o, v)	UxDDPutProp(o, XmNdirMask, ((XtArgVal)(v)))
#define	UxPutDirListLabelString(o, v)	UxDDPutProp(o, XmNdirListLabelString, ((XtArgVal)(v)))
#define	UxPutDirListItemCount(o, v)	UxPutProp(o, XmNdirListItemCount, ((XtArgVal)(v)))
#define	UxPutDirListItems(o, v)	UxDDPutProp(o, XmNdirListItems, ((XtArgVal)(v)))
#define	UxPutDirectoryValid(o, v)	UxDDPutProp(o, XmNdirectoryValid, ((XtArgVal)(v)))
#define	UxPutDirectory(o, v)	UxDDPutProp(o, XmNdirectory, ((XtArgVal)(v)))
#define	UxPutChildVerticalAlignment(o, v)	UxDDPutProp(o, XmNchildVerticalAlignment, ((XtArgVal)(v)))
#define	UxPutChildHorizontalSpacing(o, v)	UxDDPutProp(o, XmNchildHorizontalSpacing, ((XtArgVal)(v)))
#define	UxPutChildHorizontalAlignment(o, v)	UxDDPutProp(o, XmNchildHorizontalAlignment, ((XtArgVal)(v)))
#define	UxPutChildType(o, v)	UxDDPutProp(o, XmNchildType, ((XtArgVal)(v)))
#define	UxPutTopWidget(o, v)	UxDDPutProp(o, XmNtopWidget, ((XtArgVal)(v)))
#define	UxPutTopPosition(o, v)	UxPutProp(o, XmNtopPosition, ((XtArgVal)(v)))
#define	UxPutTopOffset(o, v)	UxPutProp(o, XmNtopOffset, ((XtArgVal)(v)))
#define	UxPutTopAttachment(o, v)	UxDDPutProp(o, XmNtopAttachment, ((XtArgVal)(v)))
#define	UxPutRightWidget(o, v)	UxDDPutProp(o, XmNrightWidget, ((XtArgVal)(v)))
#define	UxPutRightPosition(o, v)	UxPutProp(o, XmNrightPosition, ((XtArgVal)(v)))
#define	UxPutRightOffset(o, v)	UxPutProp(o, XmNrightOffset, ((XtArgVal)(v)))
#define	UxPutRightAttachment(o, v)	UxDDPutProp(o, XmNrightAttachment, ((XtArgVal)(v)))
#define	UxPutResizable(o, v)	UxDDPutProp(o, XmNresizable, ((XtArgVal)(v)))
#define	UxPutLeftWidget(o, v)	UxDDPutProp(o, XmNleftWidget, ((XtArgVal)(v)))
#define	UxPutLeftPosition(o, v)	UxPutProp(o, XmNleftPosition, ((XtArgVal)(v)))
#define	UxPutLeftOffset(o, v)	UxPutProp(o, XmNleftOffset, ((XtArgVal)(v)))
#define	UxPutLeftAttachment(o, v)	UxDDPutProp(o, XmNleftAttachment, ((XtArgVal)(v)))
#define	UxPutBottomWidget(o, v)	UxDDPutProp(o, XmNbottomWidget, ((XtArgVal)(v)))
#define	UxPutBottomPosition(o, v)	UxPutProp(o, XmNbottomPosition, ((XtArgVal)(v)))
#define	UxPutBottomOffset(o, v)	UxPutProp(o, XmNbottomOffset, ((XtArgVal)(v)))
#define	UxPutBottomAttachment(o, v)	UxDDPutProp(o, XmNbottomAttachment, ((XtArgVal)(v)))
#define	UxPutVerticalSpacing(o, v)	UxDDPutProp(o, XmNverticalSpacing, ((XtArgVal)(v)))
#define	UxPutRubberPositioning(o, v)	UxDDPutProp(o, XmNrubberPositioning, ((XtArgVal)(v)))
#define	UxPutHorizontalSpacing(o, v)	UxDDPutProp(o, XmNhorizontalSpacing, ((XtArgVal)(v)))
#define	UxPutFractionBase(o, v)	UxPutProp(o, XmNfractionBase, ((XtArgVal)(v)))
#define	UxPutPushButtonEnabled(o, v)	UxDDPutProp(o, XmNpushButtonEnabled, ((XtArgVal)(v)))
#define	UxPutInsertPosition(o, v)	UxPutProp(o, XmNinsertPosition, ((XtArgVal)(v)))
#define	UxPutPromptString(o, v)	UxDDPutProp(o, XmNpromptString, ((XtArgVal)(v)))
#define	UxPutHistoryVisibleItemCount(o, v)	UxPutProp(o, XmNhistoryVisibleItemCount, ((XtArgVal)(v)))
#define	UxPutHistoryMaxItems(o, v)	UxPutProp(o, XmNhistoryMaxItems, ((XtArgVal)(v)))
#define	UxPutHistoryItemCount(o, v)	UxPutProp(o, XmNhistoryItemCount, ((XtArgVal)(v)))
#define	UxPutHistoryItems(o, v)	UxDDPutProp(o, XmNhistoryItems, ((XtArgVal)(v)))
#define	UxPutCommand(o, v)	UxDDPutProp(o, XmNcommand, ((XtArgVal)(v)))
#define	UxPutSubMenuId(o, v)	UxDDPutProp(o, XmNsubMenuId, ((XtArgVal)(v)))
#define	UxPutMappingDelay(o, v)	UxPutProp(o, XmNmappingDelay, ((XtArgVal)(v)))
#define	UxPutCascadePixmap(o, v)	UxDDPutProp(o, XmNcascadePixmap, ((XtArgVal)(v)))
#define	UxPutTextTranslations(o, v)	UxDDPutProp(o, XmNtextTranslations, ((XtArgVal)(v)))
#define	UxPutTextFontList(o, v)	UxDDPutProp(o, XmNtextFontList, ((XtArgVal)(v)))
#define	UxPutShadowType(o, v)	UxDDPutProp(o, XmNshadowType, ((XtArgVal)(v)))
#define	UxPutResizePolicy(o, v)	UxDDPutProp(o, XmNresizePolicy, ((XtArgVal)(v)))
#define	UxPutNoResize(o, v)	UxDDPutProp(o, XmNnoResize, ((XtArgVal)(v)))
#define	UxPutMarginWidth(o, v)	UxDDPutProp(o, XmNmarginWidth, ((XtArgVal)(v)))
#define	UxPutMarginHeight(o, v)	UxDDPutProp(o, XmNmarginHeight, ((XtArgVal)(v)))
#define	UxPutLabelFontList(o, v)	UxDDPutProp(o, XmNlabelFontList, ((XtArgVal)(v)))
#define	UxPutDialogTitle(o, v)	UxDDPutProp(o, XmNdialogTitle, ((XtArgVal)(v)))
#define	UxPutDialogStyle(o, v)	UxDDPutProp(o, XmNdialogStyle, ((XtArgVal)(v)))
#define	UxPutDefaultPosition(o, v)	UxDDPutProp(o, XmNdefaultPosition, ((XtArgVal)(v)))
#define	UxPutDefaultButton(o, v)	UxDDPutProp(o, XmNdefaultButton, ((XtArgVal)(v)))
#define	UxPutCancelButton(o, v)	UxDDPutProp(o, XmNcancelButton, ((XtArgVal)(v)))
#define	UxPutButtonFontList(o, v)	UxDDPutProp(o, XmNbuttonFontList, ((XtArgVal)(v)))
#define	UxPutAutoUnmanage(o, v)	UxDDPutProp(o, XmNautoUnmanage, ((XtArgVal)(v)))
#define	UxPutAllowOverlap(o, v)	UxDDPutProp(o, XmNallowOverlap, ((XtArgVal)(v)))
#define	UxPutMultiClick(o, v)	UxDDPutProp(o, XmNmultiClick, ((XtArgVal)(v)))
#define	UxPutArrowDirection(o, v)	UxDDPutProp(o, XmNarrowDirection, ((XtArgVal)(v)))
#define	UxPutArgv(o, v)	UxPutProp(o, XmNargv, ((XtArgVal)(v)))
#define	UxPutArgc(o, v)	UxPutProp(o, XmNargc, ((XtArgVal)(v)))
#define	UxPutY(o, v)	UxDDPutProp(o, XmNy, ((XtArgVal)(v)))
#define	UxPutX(o, v)	UxDDPutProp(o, XmNx, ((XtArgVal)(v)))
#define	UxPutWidth(o, v)	UxDDPutProp(o, XmNwidth, ((XtArgVal)(v)))
#define	UxPutSensitive(o, v)	UxDDPutProp(o, XmNsensitive, ((XtArgVal)(v)))
#define	UxPutHeight(o, v)	UxDDPutProp(o, XmNheight, ((XtArgVal)(v)))
#define	UxPutBorderWidth(o, v)	UxDDPutProp(o, XmNborderWidth, ((XtArgVal)(v)))
#define	UxPutAncestorSensitive(o, v)	UxDDPutProp(o, XmNancestorSensitive, ((XtArgVal)(v)))
#define	UxPutMappedWhenManaged(o, v)	UxDDPutProp(o, XmNmappedWhenManaged, ((XtArgVal)(v)))
#define	UxPutColormap(o, v)	UxDDPutProp(o, XmNcolormap, ((XtArgVal)(v)))
#define	UxPutBorderPixmap(o, v)	UxDDPutProp(o, XmNborderPixmap, ((XtArgVal)(v)))
#define	UxPutBorderColor(o, v)	UxDDPutProp(o, XmNborderColor, ((XtArgVal)(v)))
#define	UxPutBackgroundPixmap(o, v)	UxDDPutProp(o, XmNbackgroundPixmap, ((XtArgVal)(v)))
#define	UxPutBackground(o, v)	UxDDPutProp(o, XmNbackground, ((XtArgVal)(v)))
#endif /* _UXPUTSMF_INCLUDED */
