/* @(#)sp_commands.h	19.1 (ESO-IPG) 02/25/03 13:48:38 */
#define C_CLEAN 	"clean/long"
#define C_INIT 		"init/long"
#define C_SAVE 		"save/long"
#define C_SEARCH 	"search/long"
#define C_IDENT 	"xident/long"
#define C_REBIN 	"rebin/long"
#define C_CALIB_LINE 	"callin/long"
#define C_CALIB_ALL 	"calib/long"
#define C_CALIB_PLOT 	"plot/long"
#define C_EXTR_GCOOR 	"gcoord/long"
#define C_EXTR_FIT_SKY 	"fit/sky"
#define C_EXTR_FIT_COSM	"skyfit/long"
#define C_EXTR_SIMPLE 	"xsimple/long"
#define C_EXTR_WEIGHTED "extract/spec"
#define C_EXTIN 	"fext/long"
#define C_INTEGR 	"integr/long"
#define C_FLUX_PLOT 	"fplot/long"
#define C_FLUX_FIT 	"ffit/long"
#define C_FLUX_RESP 	"fresp/long"
