/*
.VERSION
 090825		last modif
*/


void  WInit_widgets();
void   WChange_Midas_dir();
void   WGet_all_dirs();
int   WGet_all_files();
int   WGet_selection();
void   WProcess_table();
char* WGet_First();



