#!/bin/sh
cd $HOME/midwork
rm -f Midas_*
rm -f FORGR* 
rm -f MIDAS*.PID
rm -f RUN* 
rm -f midjo* 
rm -f idisav*
rm -f idiserv*
rm -f midtemp*
rm -f sxw*
rm -f getXdepth**
echo directory $MID_WORK cleaned...

