#! /bin/sh 
#
# get rid of all temporary files generated by Midas
# in $MID_WORK (usually ~/midwork)
# Klaus Banse ESO - SDD                100908
# 
# 100908	last modif
# 
#**************************************************************************
#
if [ -z "$MID_WORK" ]; then
   echo No $MID_WORK dir. found - we exit
   exit 3
fi

#
cd $MID_WORK
rm -f FORGR* Midas_info* idisave* midlis*.prg midsav*.coms midtab*.prg
rm -f midtemp*.prg sxw*.dat RUNNING*

# ls

echo "  "
echo $MID_WORK cleaned.
exit 0
