C @(#)aglerror.inc	19.1 (OAA-ASTRONET) 02/25/03 13:49:24
C
C HEADER : aglerror.f - Vers. 3.6.000
      INTEGER INFOTRSH     
      INTEGER WARNTRSH     
      INTEGER ERRTRSH      
      INTEGER SEVTRSH      
      INTEGER MAXERRCD     
      INTEGER AGLNOERR     
      INTEGER UNSFEATINF   
      INTEGER CLPOUTINF    
      INTEGER NOPIXINF     
      INTEGER TOOMANYINF   
      INTEGER ILLBOUWNG    
      INTEGER CMDBOVFWNG   
      INTEGER ILLCLRWNG    
      INTEGER DEVCMDWNG    
      INTEGER STRCLPWNG    
      INTEGER LOCCLPWNG    
      INTEGER ERRFILWNG    
      INTEGER ILLCODWNG    
      INTEGER ILLMRKWNG    
      INTEGER SETSYNWNG    
      INTEGER MFEOFWNG     
      INTEGER VECLNGWNG    
      INTEGER MFNOTWNG     
      INTEGER MFACTVWNG    
      INTEGER STRSTXWNG    
      INTEGER ILLPOSWNG    
      INTEGER ASPECTWNG    
      INTEGER ILLFONTWNG   
      INTEGER UNSFEATWNG   
      INTEGER VWPOUTWNG    
      INTEGER ILLSETWNG    
      INTEGER TOOMANYWNG   
      INTEGER POSCNVERR    
      INTEGER NOWNDERR     
      INTEGER ILLWNDERR    
      INTEGER NOVWPERR     
      INTEGER UNSFEATERR   
      INTEGER ILLDEVERR    
      INTEGER NOLOGERR     
      INTEGER NOARGERR     
      INTEGER VWPSELERR    
      INTEGER BUFOVFERR    
      INTEGER ILLCLPERR    
      INTEGER MFILLCDERR   
      INTEGER MFOPENERR    
      INTEGER MFCLOSERR    
      INTEGER MFWRITERR    
      INTEGER MFREADERR    
      INTEGER MFFMTERR     
      INTEGER NODEVICE     
      INTEGER FNTNUMERR    
      INTEGER FNTFILERR    
      INTEGER MEMORYERR    
      INTEGER FNTRDERR     
      INTEGER USRTRSERR    
      INTEGER MODEERR      
      INTEGER RANGEERR     
      INTEGER ILLDRVSEV    
      INTEGER DEVIOSEV     
      INTEGER DEVOPNSEV    
      INTEGER VWPOVFSEV    
      INTEGER UNUSED005    
      INTEGER CNFOPNSEV    
      INTEGER CAPOPNSEV    
      INTEGER CAPRDSEV     
      INTEGER DEVOVFSEV    
      INTEGER UNDDRVSEV    
      INTEGER CHGBUFOVSEV  
      INTEGER NOTIMPLSEV   
      INTEGER ILLCHGSEV    
      INTEGER MEMALLOCSEV  
      PARAMETER ( INFOTRSH     =	0   )
      PARAMETER ( WARNTRSH     =	100 )
      PARAMETER ( ERRTRSH      =	200 )
      PARAMETER ( SEVTRSH      =	300 )
      PARAMETER ( MAXERRCD     =	399 )
      PARAMETER ( AGLNOERR     =	(-1))
      PARAMETER ( UNSFEATINF   =	  2 )
      PARAMETER ( CLPOUTINF    =	  3 )
      PARAMETER ( NOPIXINF     =	  4 )
      PARAMETER ( TOOMANYINF   =	  5 )
      PARAMETER ( ILLBOUWNG    =	101 )
      PARAMETER ( CMDBOVFWNG   =	102 )
      PARAMETER ( ILLCLRWNG    =	103 )
      PARAMETER ( DEVCMDWNG    =	104 )
      PARAMETER ( STRCLPWNG    =	105 )
      PARAMETER ( LOCCLPWNG    =	106 )
      PARAMETER ( ERRFILWNG    =	107 )
      PARAMETER ( ILLCODWNG    =	108 )
      PARAMETER ( ILLMRKWNG    =	109 )
      PARAMETER ( SETSYNWNG    =	110 )
      PARAMETER ( MFEOFWNG     =	111 )
      PARAMETER ( VECLNGWNG    =	112 )
      PARAMETER ( MFNOTWNG     =	113 )
      PARAMETER ( MFACTVWNG    =	114 )
      PARAMETER ( STRSTXWNG    =	115 )
      PARAMETER ( ILLPOSWNG    =	116 )
      PARAMETER ( ASPECTWNG    =	117 )
      PARAMETER ( ILLFONTWNG   =	118 )
      PARAMETER ( UNSFEATWNG   =	119 )
      PARAMETER ( VWPOUTWNG    =	120 )
      PARAMETER ( ILLSETWNG    =	121 )
      PARAMETER ( TOOMANYWNG   =	201 )
      PARAMETER ( POSCNVERR    =	202 )
      PARAMETER ( NOWNDERR     =	203 )
      PARAMETER ( ILLWNDERR    =	204 )
      PARAMETER ( NOVWPERR     =	205 )
      PARAMETER ( UNSFEATERR   =	206 )
      PARAMETER ( ILLDEVERR    =	207 )
      PARAMETER ( NOLOGERR     =	208 )
      PARAMETER ( NOARGERR     =	209 )
      PARAMETER ( VWPSELERR    =	210 )
      PARAMETER ( BUFOVFERR    =	211 )
      PARAMETER ( ILLCLPERR    =	212 )
      PARAMETER ( MFILLCDERR   =	214 )
      PARAMETER ( MFOPENERR    =	215 )
      PARAMETER ( MFCLOSERR    =	216 )
      PARAMETER ( MFWRITERR    =	217 )
      PARAMETER ( MFREADERR    =	218 )
      PARAMETER ( MFFMTERR     =	219 )
      PARAMETER ( NODEVICE     =	220 )
      PARAMETER ( FNTNUMERR    =	221 )
      PARAMETER ( FNTFILERR    =	222 )
      PARAMETER ( MEMORYERR    =	223 )
      PARAMETER ( FNTRDERR     =	224 )
      PARAMETER ( USRTRSERR    =	225 )
      PARAMETER ( MODEERR      =	226 )
      PARAMETER ( RANGEERR     =	227 )
      PARAMETER ( ILLDRVSEV    =	301 )
      PARAMETER ( DEVIOSEV     =	302 )
      PARAMETER ( DEVOPNSEV    =	303 )
      PARAMETER ( VWPOVFSEV    =	304 )
      PARAMETER ( UNUSED005    =	305 )
      PARAMETER ( CNFOPNSEV    =	306 )
      PARAMETER ( CAPOPNSEV    =	307 )
      PARAMETER ( CAPRDSEV     =	308 )
      PARAMETER ( DEVOVFSEV    =	309 )
      PARAMETER ( UNDDRVSEV    =	310 )
      PARAMETER ( CHGBUFOVSEV  =	311 )
      PARAMETER ( NOTIMPLSEV   =	312 )
      PARAMETER ( ILLCHGSEV    =	313 )
      PARAMETER ( MEMALLOCSEV  =	314 )
