#include <stdlib.h>

int map(size,pntr)
int   size;      /* IN : number of data values (pixels) to be mapped */
char  **pntr;    /* OUT: pointer to data in memory */

{
char  *mypntr;


mypntr = malloc((size_t) size);   

if (mypntr == (char *) 0) return (-1);

*pntr = mypntr;
return 0;
}
