C @(#)truncy.for	19.1 (ES0-DMD) 02/25/03 13:30:49
C===========================================================================
C Copyright (C) 1995 European Southern Observatory (ESO)
C
C This program is free software; you can redistribute it and/or 
C modify it under the terms of the GNU General Public License as 
C published by the Free Software Foundation; either version 2 of 
C the License, or (at your option) any later version.
C
C This program is distributed in the hope that it will be useful,
C but WITHOUT ANY WARRANTY; without even the implied warranty of
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C GNU General Public License for more details.
C
C You should have received a copy of the GNU General Public 
C License along with this program; if not, write to the Free 
C Software Foundation, Inc., 675 Massachusetss Ave, Cambridge, 
C MA 02139, USA.
C
C Corresponding concerning ESO-MIDAS should be addressed as follows:
C	Internet e-mail: midas@eso.org
C	Postal address: European Southern Observatory
C			Data Management Division 
C			Karl-Schwarzschild-Strasse 2
C			D 85748 Garching bei Muenchen 
C			GERMANY
C===========================================================================
C
      SUBROUTINE TRUNCY(A,NPIX,TRUNC)
C +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C.AUTHOR:  K. Banse                                                             
C.VERSION: 830601  KB  Creation                                                 
C----------------------------------------------------------------------         
      IMPLICIT  NONE
      REAL      A(1)
      INTEGER   NPIX(2) 
      REAL      TRUNC(2)
C
      REAL      TMIN
      REAL      TMAX
      INTEGER   N
C                                                                               
C *** move down through frame, since we should be positioned at the end...!     
C                                                                               
      TMIN   = TRUNC(1)                                                         
      TMAX   = TRUNC(2)                                                         
C                                                                               
      DO 10 N = NPIX(1)*NPIX(2),1,-1                                            
         IF (A(N).LT.TMIN) THEN                                                 
            A(N)   = TMIN                                                       
         ELSE                                                                   
           IF (A(N).GT.TMAX) A(N)   = TMAX                                      
         END IF                                                                 
   10 CONTINUE                                                                  
C                                                                               
      RETURN                                                                    
      END

