C @(#)world.for	19.1 (ES0-DMD) 02/25/03 14:29:03
C===========================================================================
C Copyright (C) 1995 European Southern Observatory (ESO)
C
C This program is free software; you can redistribute it and/or 
C modify it under the terms of the GNU General Public License as 
C published by the Free Software Foundation; either version 2 of 
C the License, or (at your option) any later version.
C
C This program is distributed in the hope that it will be useful,
C but WITHOUT ANY WARRANTY; without even the implied warranty of
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C GNU General Public License for more details.
C
C You should have received a copy of the GNU General Public 
C License along with this program; if not, write to the Free 
C Software Foundation, Inc., 675 Massachusetss Ave, Cambridge, 
C MA 02139, USA.
C
C Corresponding concerning ESO-MIDAS should be addressed as follows:
C	Internet e-mail: midas@eso.org
C	Postal address: European Southern Observatory
C			Data Management Division 
C			Karl-Schwarzschild-Strasse 2
C			D 85748 Garching bei Muenchen 
C			GERMANY
C===========================================================================
C
C Module: world.for
C Provides functions for the manipulation of world
C coordinates in spectroscopy packages
C

C-----------------------------------------------------
        FUNCTION XI_R8R8 (X, START, STEP)

         DOUBLE PRECISION XI_R8R8
         DOUBLE PRECISION X, START(1),STEP(1)

         XI_R8R8 = (X-START(1))/step(1) + 1.D0

        RETURN
        END

C----------------------------------------------------
        FUNCTION IX_IR8(I,START,STEP)

        DOUBLE  PRECISION IX_IR8, START(1),STEP(1)
        INTEGER I

          IX_IR8 = START(1) + (I-1.)*STEP(1)

        RETURN
        END

C----------------------------------------------------
        FUNCTION IX_R8R8(I,START,STEP)

        DOUBLE  PRECISION I, IX_R8R8, START(1),STEP(1)

          IX_R8R8 = START(1) + (I-1.)*STEP(1)

        RETURN
        END

