/* @(#)UimxResources.h	1.1 (ESO-IPG) 4/14/93 16:05:08 */

#define    TextFont         "-Adobe-helvetica-medium-r-normal--14*"
#define    BoldTextFont     "-Adobe-helvetica-bold-r-normal--14*"
#define    BigFont          "12x24"
#define    BoldBigFont      "12x24"
#define    SmallFont        "8x13"
#define    BoldSmallFont    "8x13bold"
#define    SmallPropFont     "-Adobe-helvetica-bold-r-normal--12*"

#define    TextForeground   "Black"
#define    TextBackground   "White"
#define    LabelBackground  "Gray80"
#define    WindowBackground "Gray80"
#define    ApplicBackground "Gray80"
#define    SHelpBackground  "FloralWhite"
#define    MenuBackground   "DeepSkyBlue3"
#define    MenuForeground   "White"
#define    ListBackground   "Grey95"
#define    ListForeground   "Black"

#define    ButtonBackground "Gray70"
#define    ButtonForeground "NavyBlue"
#define    CancelForeground "yellow"
#define    ApplyForeground  "OrangeRed2"
#define    SelectColor      "green"

#define    WindowGeometry   "+20+340"
#define    ExtHelpGeometry  "+20+340"
#define    SelBoxGeometry   "+20+340"
