#! /bin/sh
# @(#)idigo.sh	19.1 (ESO-IPG) 02/25/03 14:31:43
# 
# K. Banse	ESO - Garching
# 
DAZUNIT=AX
WORK_STATION=DAZAX
I_TYPE="SXW  " 
MID_WORK=$HOME/midwork/
MID_SYSTAB=$MIDASHOME/$MIDVERS/systab/bin
export DAZUNIT WORK_STATION I_TYPE MID_WORK MID_SYSTAB
$MIDASHOME/$MIDVERS/system/exec/idiserv.exe 
exit

