C  @(#)dlsq.inc	19.1 (ESO-IPG) 02/25/03 13:28:38
C
C	Copyright (C) Andrew T. Young, 1990
C
C       Declares arrays for DLSQ.
C
C
      DOUBLE PRECISION YT,A,SP,P,PART,WVAR,DUM,YC,SCALE
      REAL PG,X,Z,W,Y
      INTEGER KDUM,IX,KIP,IP,NEQS,MAXP,KIP9,KZ,KX,NONEQ,NPT,IDF,IT,IPR
      INTEGER MAXIT,MAXK,MAXS,MAXN,NPNITE,KBASEN,KBASEW
C	N.B.: ALL variables appearing here MUST have explicit types !!!
      COMMON /LEASTC/ KDUM(24),IX(MXP),YC(MXPT),Y(MXPT),YT,W(MXPT),KIP,
     1 NEQS,IP(MXP),A(MXP*(MXP+1)/2),SP(MXP),PG(MXP),P(MXP),PART(MXP),
     2 DUM(16),MAXP,KIP9,
     3 KZ(MKX),Z(MXX),X(MXX,MXPT),KX(MKX,MXPT),WVAR,SCALE,
     4 NONEQ,NPT,IDF,IT,MAXIT,IPR
C    (RE-ORDERED FOR BASE-ADDRESS EFFICIENCY.)
      EQUIVALENCE (KDUM(1),MAXK), (KDUM(2),MAXS), (KDUM(3),MAXN),
     1		(KDUM(4),NPNITE),
     2		(KDUM(14),KBASEN), (KDUM(15),KBASEW)
