C  @(#)obs.inc	19.1 (ESO-IPG) 02/25/03 13:28:38
C
C     observational data storage:
C
      INTEGER MXOBS,MXNITE
C
      PARAMETER (MXOBS=50000)
C       MXOBS is max.number of observational data.
C
      PARAMETER (MXNITE=30)
C       MXNITE is max.number of nights.
C
      DOUBLE PRECISION DTZERO
      REAL DJOBS,AIRM,SIGNAL,EXPTIM,AZIM,FTMP,DTMP,RELH,PRESS,DOME
      REAL GENQ,GENR,GENG,ESTER,XS,YS,PHVAR
      INTEGER NSTR,NITE,KBND,NIGHTS,ISTARS,NDATA
C
C   Common for data:
C
      COMMON/DATA1/DJOBS(MXOBS),AIRM(MXOBS),SIGNAL(MXOBS),EXPTIM(MXOBS),
     1          AZIM(MXOBS),FTMP(MXOBS),DTMP(MXOBS),RELH(MXOBS),
     2     PRESS(MXOBS),DOME(MXOBS),GENQ(MXOBS),GENR(MXOBS),GENG(MXOBS),
     3          ESTER(MXOBS), XS(MXOBS),YS(MXOBS), DTZERO(MXNITE),
     4          PHVAR(MXOBS),
     5          NSTR(MXOBS),NITE(MXOBS),KBND(MXOBS), NIGHTS,ISTARS,NDATA
C
      CHARACTER*4 DIAPH(MXOBS),PMTV(MXOBS),NDFILT(MXOBS),SEEING(MXOBS)
      CHARACTER SYMB(MXOBS)
      COMMON /DATA2/ DIAPH,PMTV,NDFILT,SEEING,SYMB
