/*===========================================================================
  Copyright (C) 1995-2009 European Southern Observatory (ESO)
 
  This program is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public 
  License along with this program; if not, write to the Free 
  Software Foundation, Inc., 675 Massachusetts Ave, Cambridge, 
  MA 02139, USA.
 
  Correspondence concerning ESO-MIDAS should be addressed as follows:
	Internet e-mail: midas@eso.org
	Postal address: European Southern Observatory
			Data Management Division 
			Karl-Schwarzschild-Strasse 2
			D 85748 Garching bei Muenchen 
			GERMANY
.VERSION
 090826		last modif

===========================================================================*/


#include <math.h>
#include <stdio.h>
#include <string.h>
#include <UxLib.h>

void	popup_flx(fluxfile)
char	fluxfile[];
{
	UxPutText(UxFindSwidget("textAsk"),fluxfile);
	UxPutTitle(UxFindSwidget("AskShell"),"Modify_the_Flux_filename");
	UxPutLabelString(UxFindSwidget("labelAsk"),"Flux file (no extension):");
	UxPopupInterface(UxFindSwidget("AskShell"),exclusive_grab);
}
