C @(#)lisib.for	19.1 (ES0-DMD) 02/25/03 13:29:44
C===========================================================================
C Copyright (C) 1995 European Southern Observatory (ESO)
C
C This program is free software; you can redistribute it and/or 
C modify it under the terms of the GNU General Public License as 
C published by the Free Software Foundation; either version 2 of 
C the License, or (at your option) any later version.
C
C This program is distributed in the hope that it will be useful,
C but WITHOUT ANY WARRANTY; without even the implied warranty of
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C GNU General Public License for more details.
C
C You should have received a copy of the GNU General Public 
C License along with this program; if not, write to the Free 
C Software Foundation, Inc., 675 Massachusetss Ave, Cambridge, 
C MA 02139, USA.
C
C Corresponding concerning ESO-MIDAS should be addressed as follows:
C	Internet e-mail: midas@eso.org
C	Postal address: European Southern Observatory
C			Data Management Division 
C			Karl-Schwarzschild-Strasse 2
C			D 85748 Garching bei Muenchen 
C			GERMANY
C===========================================================================
C
      SUBROUTINE LISIB(RMT,VC,NIN,NCD,SIGP)
C
C++++
C
C----
      IMPLICIT   NONE
      INTEGER    NCD
      REAL       RMT(NCD,NCD)
      REAL       VC(NCD)
      INTEGER    NIN	
      REAL       SIGP(1)
C
      REAL       DEL(1000)
      INTEGER    I, J, K
      REAL       DET
C 
C ***
      DO 10 I = 1,NIN
         DEL(I) = 0
   10 CONTINUE
C
      CALL SMAIN(RMT,NCD,NIN,DET)
      DO 20 J = 1,NIN
         DO 21 K = 1,NIN
            DEL(J) = DEL(J)+VC(K)*RMT(J,K)
   21    CONTINUE
   20 CONTINUE
C
      DO 30 J=1,NIN
         VC(J)   = DEL(J)
         SIGP(J) = SQRT(RMT(J,J))
   30 CONTINUE
      
      RETURN
      END
