/*===========================================================================
  Copyright (C) 1995-2009 European Southern Observatory (ESO)
 
  This program is free software; you can redistribute it and/or 
  modify it under the terms of the GNU General Public License as 
  published by the Free Software Foundation; either version 2 of 
  the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public 
  License along with this program; if not, write to the Free 
  Software Foundation, Inc., 675 Massachusetss Ave, Cambridge, 
  MA 02139, USA.
 
  Corresponding concerning ESO-MIDAS should be addressed as follows:
	Internet e-mail: midas@eso.org
	Postal address: European Southern Observatory
			Data Management Division 
			Karl-Schwarzschild-Strasse 2
			D 85748 Garching bei Muenchen 
			GERMANY
.VERSION
 090826         last modif

===========================================================================*/


#include <stdio.h>
#include <string.h>
#include <globaldef.h>
#include <model.h>


extern void endname();





void	save_curve(name)
char	name[];
{
extern	struct s_trans	T;
extern	float   *Sxx,*Syy;
extern	int     Snp;
extern	char	Sstr[];

register int	i;
FILE	*out;
char	term[10];

if(strlen(name)!=0)
{
endname(name,term);
if(strcmp(term,"dat"))
	strcat(name,".dat");
}
else
	strcpy(name,"tmp.dat");

strcpy(T.curvefile,name);

out=fopen(name,"w");

fprintf(out,"# curve data: %s\n",name);
fprintf(out,"%s\n",Sstr);

for(i=0;i<Snp;i++)
	fprintf(out,"%f %e\n",Sxx[i],Syy[i]);

fclose(out);
}
