.\" @(#)helpmidas.l	19.1 (ESO-IPG) 02/25/03 14:32:14
.\" Copyright (c) 1989 European Southern Observatory.
.\" All rights reserved.  
.\"
.TH helpmidas 1L "07 July 1993" MIDAS
.SH NAME
helpmidas \- Standalone GUI help for MIDAS.
.SH SYNOPSIS
.B helpmidas
.br 
[
.B \-h
.I midashome
] [
.B \-r
.I midvers
] 
.br 
[
.B \-d
.I display
] [
.B \-m
.I mid_work
] 
.SH DESCRIPTION
.I helpmidas 
starts a standalone GUI help for MIDAS. 
.PP
Without arguments,
.I helpmidas 
initiates a MIDAS help session with default definitions.
Some of these definitions can be modified with arguments in the command line of
.I helpmidas
or by environment variables. Arguments in the command
line are taken with preference to environment variables.
.SH OPTIONS
.I helpmidas 
has been configured by the operator at installation time to start the binary
XHelp.exe located in 
.IR $MIDASHOME0/$MIDVERS0/gui/exec directory
(defined internally in the script).
However alternative releases can be specified using the:
.IP "\fB-h\fp \fImidashome\fp"
Home directory for MIDAS. Absolute pathname containing, at least, one release 
of MIDAS. It may also contain subdirectories for demo and calibration data.
.IP "\fB-r\fp \fImidvers\fp"
Release of MIDAS to be executed. It must be a subdirectory under 
.IR midashome.
.IP "\fB-d\fp \fIdisplay\fp"
Specifies another X server for the display and graphical MIDAS windows (
.B NOTE:
be aware of allowed access to a remote X server using the
.IR xhost (1)
command.)
.IP "\fB-m\fp \fImid_work\fp"
Specifies the MIDAS startup directory (default: $HOME/midwork). This GUI 
will create this directory if it does not exist and uses in to keep some 
temporary files. 
.SH ENVIRONMENT
.I helpmidas
can also use the following definitions in 
the environment with the same performance as previous described arguments.
.br
.IP "\fBMIDASHOME\fp"
Home directory for MIDAS.
.IP "\fBMIDVERS\fp"
Release of MIDAS.
.IP "\fBDISPLAY\fp"
X server for the display and graphical MIDAS windows.
.IP "\fBMID_WORK\fp"
Startup directory for MIDAS.
.SH SEE ALSO
.IR inmidas(1L),gomidas(1L)
