C @(#)lorentz.for	19.1 (ES0-DMD) 02/25/03 13:17:40
C===========================================================================
C Copyright (C) 1995 European Southern Observatory (ESO)
C
C This program is free software; you can redistribute it and/or 
C modify it under the terms of the GNU General Public License as 
C published by the Free Software Foundation; either version 2 of 
C the License, or (at your option) any later version.
C
C This program is distributed in the hope that it will be useful,
C but WITHOUT ANY WARRANTY; without even the implied warranty of
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C GNU General Public License for more details.
C
C You should have received a copy of the GNU General Public 
C License along with this program; if not, write to the Free 
C Software Foundation, Inc., 675 Massachusetss Ave, Cambridge, 
C MA 02139, USA.
C
C Corresponding concerning ESO-MIDAS should be addressed as follows:
C	Internet e-mail: midas@eso.org
C	Postal address: European Southern Observatory
C			Data Management Division 
C			Karl-Schwarzschild-Strasse 2
C			D 85748 Garching bei Muenchen 
C			GERMANY
C===========================================================================
C
      SUBROUTINE LORENZ(INDEP,X,NP,PARAM,Y,DERIV)                              
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++                  
C.COPYRIGHT: Copyright (c) 1987 European Southern Observatory,                  
C                                         all rights reserved                   
C                                                                               
C.VERSION: 1.0  ESO-FORTRAN Conversion, AA  18:01 - 21 DEC 1987                 
C                                                                               
C.LANGUAGE: F77+ESOext                                                          
C                                                                               
C.AUTHOR: J.D.PONZ                                                              
C                                                                               
C--------------------------------------------------------------                 
C***************************************************                            
C                                                                               
C      IMPLICIT NONE                                                            
C  FIT WITH A (MODIFIED) LORENTZ-DISTRIBUTION                                   
C                                                                               
C   Y = P1/(1+(2(X-P2)/P3)**2)**P4                                              
C                                                                               
C.AUTHOR   O. RICHTER                                                           
C.MODIFICATION                                                                  
C    20 AUG 87        FM PHD     (error y1 <-> y + factor of 2)                 
C                                                                               
      INTEGER NP,INDEP
      DOUBLE PRECISION Y,PARAM(NP),DERIV(NP)                                    
      REAL X(INDEP)                                                        
      DOUBLE PRECISION B,C,D,E                                                
C                                                                               
      B      = 2.D0* (X(1)-PARAM(2))/PARAM(3)                                   
      C      = B*B                                                              
      D      = 1.0D0 + C                                                        
      E      = D** (-PARAM(4))                                                  
      DERIV(1) = E                                                              
      Y      = PARAM(1)*E                                                       
      DERIV(2) = 4.0D0*PARAM(4)*B*Y/D/PARAM(3)                                  
C      DERIV(3) = 2.D0*DERIV(2)*B                                               
C      DERIV(4) = -E*DLOG(D)                                                    
      DERIV(3) = 0.5D0*DERIV(2)*B                                               
      DERIV(4) = -Y*DLOG(D)                                                     
      RETURN                                                                    
                                                                                
      END                                                                       
