C===========================================================================
C Copyright (C) 1995-2006 European Southern Observatory (ESO)
C
C This program is free software; you can redistribute it and/or 
C modify it under the terms of the GNU General Public License as 
C published by the Free Software Foundation; either version 2 of 
C the License, or (at your option) any later version.
C
C This program is distributed in the hope that it will be useful,
C but WITHOUT ANY WARRANTY; without even the implied warranty of
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C GNU General Public License for more details.
C
C You should have received a copy of the GNU General Public 
C License along with this program; if not, write to the Free 
C Software Foundation, Inc., 675 Massachusetts Ave, Cambridge, 
C MA 02139, USA.
C
C Correspondence concerning ESO-MIDAS should be addressed as follows:
C	Internet e-mail: midas@eso.org
C	Postal address: European Southern Observatory
C			Data Management Division 
C			Karl-Schwarzschild-Strasse 2
C			D 85748 Garching bei Muenchen 
C			GERMANY
C===========================================================================

C ++++++++++++++++++++++++  ytbr.for +++++++++++++++++++++++++++++++++++++++
C .LANGUAGE Fortran 77
C .IDENTIFICATION Module ytbr.foR
C  AUTHOR         K. Banse
C  Module contains layer between the table related FORTRAN TBxxxx interfaces
C  and the TC_interfaces written in (hopefully independent) C
C .KEYWORDS       standard interfaces.
C .ENVIRONMENT    FORTRAN and C standards
C .VERSION  [1.00] 871207:  created from SXFTOC.C
C 
C 060407	last modif
C -----------------------------------------------------------------------------
C
      SUBROUTINE TBRRDC(TID, ROW, NC, COLS, VALUE, NULL, STATUS)
C
      IMPLICIT NONE
C
      CHARACTER*(*)   VALUE
      INTEGER     TID, ROW, NC, COLS, NULL
      INTEGER     STATUS
C 
      CALL STLOC(1,1,VALUE)                              !blanked CHAR_LOC
C 
      CALL TBR1(TID,ROW,NC,COLS,NULL,STATUS)
C
      RETURN
      END
C 
C 
      SUBROUTINE TBRGET(TID, CTYPE, ROW, NC, COLS, VALUE, NULL, STATUS)
C
      IMPLICIT NONE
C
      CHARACTER*(*)   CTYPE
      INTEGER     TID, ROW, NC, COLS, NULL
      INTEGER     VALUE(*)
      INTEGER     STATUS
C
      CALL STSTR(1,CTYPE)                     !STRIPPED_STRING
C 
      CALL TBR3(TID,ROW,NC,COLS,VALUE,NULL,STATUS)
C
      RETURN
      END
C
C
      SUBROUTINE TBRPUT(TID, CTYPE, ROW, NC, COLS, VALUE, STATUS)
C
      IMPLICIT NONE
C
      CHARACTER*(*)   CTYPE
      INTEGER     TID, ROW, NC, COLS
      INTEGER     VALUE(*)
      INTEGER     STATUS
C
      CALL STSTR(1,CTYPE)                     !STRIPPED_STRING
C
      CALL TBR4(TID,ROW,NC,COLS,VALUE,STATUS)
C
      RETURN
      END
C 
C
      SUBROUTINE TBRWRC(TID, ROW, NC, COLS, VALUE, STATUS)
C
      IMPLICIT NONE
C
      CHARACTER*(*)   VALUE
      INTEGER     TID, ROW, NC, COLS
      INTEGER     STATUS
C
      CALL STLOC(1,0,VALUE)                              !untouched CHAR_LOC
C 
      CALL TBR2(TID,ROW,NC,COLS,STATUS)
C
      RETURN
      END
CC
CC all other TBRxyz functions
CC handled directly in tbr.fc


