C @(#)blinkxy.for	19.1 (ES0-DMD) 02/25/03 13:23:04
C===========================================================================
C Copyright (C) 1995 European Southern Observatory (ESO)
C
C This program is free software; you can redistribute it and/or 
C modify it under the terms of the GNU General Public License as 
C published by the Free Software Foundation; either version 2 of 
C the License, or (at your option) any later version.
C
C This program is distributed in the hope that it will be useful,
C but WITHOUT ANY WARRANTY; without even the implied warranty of
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C GNU General Public License for more details.
C
C You should have received a copy of the GNU General Public 
C License along with this program; if not, write to the Free 
C Software Foundation, Inc., 675 Massachusetss Ave, Cambridge, 
C MA 02139, USA.
C
C Corresponding concerning ESO-MIDAS should be addressed as follows:
C	Internet e-mail: midas@eso.org
C	Postal address: European Southern Observatory
C			Data Management Division 
C			Karl-Schwarzschild-Strasse 2
C			D 85748 Garching bei Muenchen 
C			GERMANY
C===========================================================================
C
C @(#)blinkxy.for	19.1 (ESO-SDAG) 02/25/03 13:23:04
      SUBROUTINE BLINKXY(XBLINK,YBLINK,XCUR,YCUR,ITYP)
C
C TRANSFORMS BLINK COORDINATES INTO (+,-) MICRONS (ITYP=1) OR
C VICE VERSA (ITYP = 2). THE BLINK HAS 0.5 MICRON STEP AND
C NEGATIVE VALUES ARE REPRESENTED BY (10000000 + VALUE)
C
      IMPLICIT REAL*8 (A-H,O-Z), INTEGER*4 (I-N)
C
C BLINK COORDINATES INTO (+,-) MICRONS
C
      IF (ITYP.EQ.1) THEN
         IF (XBLINK.GT.5000000.) XBLINK = XBLINK -10000000.
         IF (YBLINK.GT.5000000.) YBLINK = YBLINK -10000000.
         XCUR = XBLINK/2.
         YCUR = YBLINK/2.
      ELSE
         XBLINK = XCUR*2.
         YBLINK = YCUR*2.
         IF (XBLINK.LT.0) XBLINK = XBLINK + 10000000.
         IF (YBLINK.LT.0) YBLINK = YBLINK + 10000000.
      ENDIF
      RETURN
      END   
