C @(#)sort.for	19.1 (ES0-DMD) 02/25/03 13:32:58
C===========================================================================
C Copyright (C) 1995 European Southern Observatory (ESO)
C
C This program is free software; you can redistribute it and/or 
C modify it under the terms of the GNU General Public License as 
C published by the Free Software Foundation; either version 2 of 
C the License, or (at your option) any later version.
C
C This program is distributed in the hope that it will be useful,
C but WITHOUT ANY WARRANTY; without even the implied warranty of
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C GNU General Public License for more details.
C
C You should have received a copy of the GNU General Public 
C License along with this program; if not, write to the Free 
C Software Foundation, Inc., 675 Massachusetss Ave, Cambridge, 
C MA 02139, USA.
C
C Corresponding concerning ESO-MIDAS should be addressed as follows:
C	Internet e-mail: midas@eso.org
C	Postal address: European Southern Observatory
C			Data Management Division 
C			Karl-Schwarzschild-Strasse 2
C			D 85748 Garching bei Muenchen 
C			GERMANY
C===========================================================================
C
C  @(#)sort.for	5.1 (ESO-IPG) 4/5/93 15:58:34
      SUBROUTINE SORT(N,X)
      INTEGER N
      DOUBLE PRECISION X(1),SAVE
C
      INTEGER M,J,I
C
C ... shell sort
C
      M=-N
 2      M=M/2
        IF (M.LT.0) THEN
          DO 3 J=1,N+M
            DO 1 I=J,1,M
              IF (X(I).LE.X(I-M)) THEN
                GOTO 3
              ELSE
                SAVE=X(I)
                X(I)=X(I-M)
                X(I-M)=SAVE
              ENDIF
 1          CONTINUE
 3        CONTINUE
          GOTO 2
        ENDIF
      END
