\def\eos#1{\EOF}%			End of Screen
\def\ul#1{{\Vu#1}}%                     Underline
\def\LeftLine#1{\leftline{#1}}%         Synonym
\def\RightLine#1{\rightline{#1}}%       Synonym
\def\CentreLine#1{\centerline{#1}}%     Synonym
\def\NewWindow#1{\CheckLines{#1}}%
\def\BeginLeft{\begin{left}}%
\def\BeginRight{\begin{right}}%
\def\BeginCentre{\begin{center}}%
\def\BeginQuote{\begin{quote}}%
\def\BeginItems{\begin{itemize}}%
\def\BeginIndent{\begin{indent}}%
\def\BeginTable#1#2{\begin{table}{#1}{#2}}%
\def\EndLeft{\end{left}}%
\def\EndRight{\end{right}}%
\def\EndCentre{\end{center}}%
\def\EndQuote{\end{quote}}%
\def\EndItems{\end{itemize}}%
\def\EndIndent{\end{itemize}}%
\def\EndTable{\end{table}}%
\def\Item{\item}%
\def\LastRule{\Rule}%
\def\CentreColumns#1#2{\multicolumn{#1}{c}{#2}}%
\def\LeftColumns#1#2{\multicolumn{#1}{l}{#2}}%
\def\RightColumns#1#2{\multicolumn{#1}{r}{#2}}%
\def\Centre#1{\multicolumn{1}{c}{#1}}%
\def\Left#1{\multicolumn{1}{l}{#1}}%
\def\Right#1{\multicolumn{1}{r}{#1}}%
\def\EndOfText{\EOF}
\def\frametext#1{\begin{table}{| c |}{\len{#1}}%
	\Rule #1 \\ \Rule  \end{table}}
