/* @(#)UxGetsMF.h	17.1 (ESO-IPG) 01/25/02 17:26:00 */
#ifndef	_UXGETSMF_INCLUDED
#define	_UXGETSMF_INCLUDED
#define    UxGetWmTimeout(o)       (int)UxGetProp(o, XmNwmTimeout)
#define    UxGetWinGravity(o)       (char *)UxDDGetProp(o, XmNwinGravity)
#define    UxGetWindowGroup(o)       (char *)UxDDGetProp(o, XmNwindowGroup)
#define    UxGetWidthInc(o)       (int)UxGetProp(o, XmNwidthInc)
#define    UxGetWaitForWm(o)       (char *)UxDDGetProp(o, XmNwaitForWm)
#define    UxGetTransient(o)       (char *)UxDDGetProp(o, XmNtransient)
#define    UxGetTitleEncoding(o)       (char *)UxDDGetProp(o, XmNtitleEncoding)
#define    UxGetTitle(o)       (char *)UxGetProp(o, XmNtitle)
#define    UxGetMinWidth(o)       (int)UxGetProp(o, XmNminWidth)
#define    UxGetMinHeight(o)       (int)UxGetProp(o, XmNminHeight)
#define    UxGetMinAspectY(o)       (int)UxGetProp(o, XmNminAspectY)
#define    UxGetMinAspectX(o)       (int)UxGetProp(o, XmNminAspectX)
#define    UxGetMaxWidth(o)       (int)UxGetProp(o, XmNmaxWidth)
#define    UxGetMaxHeight(o)       (int)UxGetProp(o, XmNmaxHeight)
#define    UxGetMaxAspectY(o)       (int)UxGetProp(o, XmNmaxAspectY)
#define    UxGetMaxAspectX(o)       (int)UxGetProp(o, XmNmaxAspectX)
#define    UxGetInput(o)       (char *)UxDDGetProp(o, XmNinput)
#define    UxGetInitialState(o)       (char *)UxDDGetProp(o, XmNinitialState)
#define    UxGetIconY(o)       (int)UxGetProp(o, XmNiconY)
#define    UxGetIconX(o)       (int)UxGetProp(o, XmNiconX)
#define    UxGetIconWindow(o)       (char *)UxDDGetProp(o, XmNiconWindow)
#define    UxGetIconPixmap(o)       (char *)UxDDGetProp(o, XmNiconPixmap)
#define    UxGetIconMask(o)       (char *)UxDDGetProp(o, XmNiconMask)
#define    UxGetHeightInc(o)       (int)UxGetProp(o, XmNheightInc)
#define    UxGetBaseWidth(o)       (int)UxGetProp(o, XmNbaseWidth)
#define    UxGetBaseHeight(o)       (int)UxGetProp(o, XmNbaseHeight)
#define    UxGetUseAsyncGeometry(o)       (char *)UxDDGetProp(o, XmNuseAsyncGeometry)
#define    UxGetShellUnitType(o)       (char *)UxDDGetProp(o, XmNshellUnitType)
#define    UxGetPreeditType(o)       (char *)UxGetProp(o, XmNpreeditType)
#define    UxGetMwmMenu(o)       (char *)UxGetProp(o, XmNmwmMenu)
#define    UxGetMwmInputMode(o)       (char *)UxDDGetProp(o, XmNmwmInputMode)
#define    UxGetMwmFunctions(o)       (int)UxGetProp(o, XmNmwmFunctions)
#define    UxGetMwmDecorations(o)       (int)UxGetProp(o, XmNmwmDecorations)
#define    UxGetKeyboardFocusPolicy(o)       (char *)UxDDGetProp(o, XmNkeyboardFocusPolicy)
#define    UxGetInputMethod(o)       (char *)UxGetProp(o, XmNinputMethod)
#define    UxGetDeleteResponse(o)       (char *)UxDDGetProp(o, XmNdeleteResponse)
#define    UxGetAudibleWarning(o)       (char *)UxDDGetProp(o, XmNaudibleWarning)
#define    UxGetTransientFor(o)       (char *)UxDDGetProp(o, XmNtransientFor)
#define    UxGetIconNameEncoding(o)       (char *)UxDDGetProp(o, XmNiconNameEncoding)
#define    UxGetIconName(o)       (char *)UxGetProp(o, XmNiconName)
#define    UxGetIconic(o)       (char *)UxDDGetProp(o, XmNiconic)
#define    UxGetVisibleWhenOff(o)       (char *)UxDDGetProp(o, XmNvisibleWhenOff)
#define    UxGetSet(o)       (char *)UxDDGetProp(o, XmNset)
#define    UxGetSelectPixmap(o)       (char *)UxDDGetProp(o, XmNselectPixmap)
#define    UxGetSelectInsensitivePixmap(o)       (char *)UxDDGetProp(o, XmNselectInsensitivePixmap)
#define    UxGetSelectColor(o)       (char *)UxDDGetProp(o, XmNselectColor)
#define    UxGetIndicatorType(o)       (char *)UxDDGetProp(o, XmNindicatorType)
#define    UxGetIndicatorSize(o)       (int)UxDDGetProp(o, XmNindicatorSize)
#define    UxGetIndicatorOn(o)       (char *)UxDDGetProp(o, XmNindicatorOn)
#define    UxGetFillOnSelect(o)       (char *)UxDDGetProp(o, XmNfillOnSelect)
#define    UxGetWordWrap(o)       (char *)UxDDGetProp(o, XmNwordWrap)
#define    UxGetRows(o)       (short)UxShortGetProp(o, XmNrows)
#define    UxGetCursorPositionVisible(o)       (char *)UxDDGetProp(o, XmNcursorPositionVisible)
#define    UxGetColumns(o)       (short)UxShortGetProp(o, XmNcolumns)
#define    UxGetBlinkRate(o)       (int)UxGetProp(o, XmNblinkRate)
#define    UxGetSource(o)       (XmTextSource)UxGetProp(o, XmNsource)
#define    UxGetSelectThreshold(o)       (int)UxGetProp(o, XmNselectThreshold)
#define    UxGetSelectionArrayCount(o)       (int)UxGetProp(o, XmNselectionArrayCount)
#define    UxGetSelectionArray(o)       (char *)UxDDGetProp(o, XmNselectionArray)
#define    UxGetPendingDelete(o)       (char *)UxDDGetProp(o, XmNpendingDelete)
#define    UxGetVerifyBell(o)       (char *)UxDDGetProp(o, XmNverifyBell)
#define    UxGetValueWcs(o)       (char *)UxDDGetProp(o, XmNvalueWcs)
#define    UxGetTopCharacter(o)       (int)UxGetProp(o, XmNtopCharacter)
#define    UxGetMaxLength(o)       (int)UxGetProp(o, XmNmaxLength)
#define    UxGetEditMode(o)       (char *)UxDDGetProp(o, XmNeditMode)
#define    UxGetEditable(o)       (char *)UxDDGetProp(o, XmNeditable)
#define    UxGetCursorPosition(o)       (int)UxGetProp(o, XmNcursorPosition)
#define    UxGetAutoShowCursorPosition(o)       (char *)UxDDGetProp(o, XmNautoShowCursorPosition)
#define    UxGetVisual(o)       (Visual *)UxGetProp(o, XmNvisual)
#define    UxGetSaveUnder(o)       (char *)UxDDGetProp(o, XmNsaveUnder)
#define    UxGetOverrideRedirect(o)       (char *)UxDDGetProp(o, XmNoverrideRedirect)
#define    UxGetGeometry(o)       (char *)UxDDGetProp(o, XmNgeometry)
#define    UxGetCreatePopupChildProc(o)       (void (*)())UxGetProp(o, XmNcreatePopupChildProc)
#define    UxGetSeparatorType(o)       (char *)UxDDGetProp(o, XmNseparatorType)
#define    UxGetMargin(o)       (int)UxDDGetProp(o, XmNmargin)
#define    UxGetTextString(o)       (char *)UxDDGetProp(o, XmNtextString)
#define    UxGetTextColumns(o)       (short)UxShortGetProp(o, XmNtextColumns)
#define    UxGetSelectionLabelString(o)       (char *)UxDDGetProp(o, XmNselectionLabelString)
#define    UxGetMustMatch(o)       (char *)UxDDGetProp(o, XmNmustMatch)
#define    UxGetListVisibleItemCount(o)       (int)UxGetProp(o, XmNlistVisibleItemCount)
#define    UxGetListLabelString(o)       (char *)UxDDGetProp(o, XmNlistLabelString)
#define    UxGetListItemCount(o)       (int)UxGetProp(o, XmNlistItemCount)
#define    UxGetListItems(o)       (char *)UxDDGetProp(o, XmNlistItems)
#define    UxGetDialogType(o)       (char *)UxDDGetProp(o, XmNdialogType)
#define    UxGetChildPlacement(o)       (char *)UxDDGetProp(o, XmNchildPlacement)
#define    UxGetApplyLabelString(o)       (char *)UxDDGetProp(o, XmNapplyLabelString)
#define    UxGetWorkWindow(o)       (char *)UxDDGetProp(o, XmNworkWindow)
#define    UxGetVisualPolicy(o)       (char *)UxDDGetProp(o, XmNvisualPolicy)
#define    UxGetScrollingPolicy(o)       (char *)UxDDGetProp(o, XmNscrollingPolicy)
#define    UxGetScrolledWindowMarginWidth(o)       (int)UxDDGetProp(o, XmNscrolledWindowMarginWidth)
#define    UxGetScrolledWindowMarginHeight(o)       (int)UxDDGetProp(o, XmNscrolledWindowMarginHeight)
#define    UxGetScrollBarPlacement(o)       (char *)UxDDGetProp(o, XmNscrollBarPlacement)
#define    UxGetClipWindow(o)       (char *)UxDDGetProp(o, XmNclipWindow)
#define    UxGetTroughColor(o)       (char *)UxDDGetProp(o, XmNtroughColor)
#define    UxGetSliderSize(o)       (int)UxGetProp(o, XmNsliderSize)
#define    UxGetShowArrows(o)       (char *)UxDDGetProp(o, XmNshowArrows)
#define    UxGetRepeatDelay(o)       (int)UxGetProp(o, XmNrepeatDelay)
#define    UxGetPageIncrement(o)       (int)UxGetProp(o, XmNpageIncrement)
#define    UxGetInitialDelay(o)       (int)UxGetProp(o, XmNinitialDelay)
#define    UxGetIncrement(o)       (int)UxGetProp(o, XmNincrement)
#define    UxGetValue(o)       (int)UxGetProp(o, XmNvalue)
#define    UxGetTitleString(o)       (char *)UxDDGetProp(o, XmNtitleString)
#define    UxGetShowValue(o)       (char *)UxDDGetProp(o, XmNshowValue)
#define    UxGetScaleWidth(o)       (int)UxDDGetProp(o, XmNscaleWidth)
#define    UxGetScaleMultiple(o)       (int)UxGetProp(o, XmNscaleMultiple)
#define    UxGetScaleHeight(o)       (int)UxDDGetProp(o, XmNscaleHeight)
#define    UxGetProcessingDirection(o)       (char *)UxDDGetProp(o, XmNprocessingDirection)
#define    UxGetMinimum(o)       (int)UxGetProp(o, XmNminimum)
#define    UxGetMaximum(o)       (int)UxGetProp(o, XmNmaximum)
#define    UxGetDecimalPoints(o)       (short)UxShortGetProp(o, XmNdecimalPoints)
#define    UxGetScrollVertical(o)       (char *)UxDDGetProp(o, XmNscrollVertical)
#define    UxGetScrollTopSide(o)       (char *)UxDDGetProp(o, XmNscrollTopSide)
#define    UxGetScrollLeftSide(o)       (char *)UxDDGetProp(o, XmNscrollLeftSide)
#define    UxGetScrollHorizontal(o)       (char *)UxDDGetProp(o, XmNscrollHorizontal)
#define    UxGetVerticalScrollBar(o)       (char *)UxDDGetProp(o, XmNverticalScrollBar)
#define    UxGetHorizontalScrollBar(o)       (char *)UxDDGetProp(o, XmNhorizontalScrollBar)
#define    UxGetWhichButton(o)       (int)UxGetProp(o, XmNwhichButton)
#define    UxGetTearOffModel(o)       (char *)UxDDGetProp(o, XmNtearOffModel)
#define    UxGetRowColumnType(o)       (char *)UxDDGetProp(o, XmNrowColumnType)
#define    UxGetResizeWidth(o)       (char *)UxDDGetProp(o, XmNresizeWidth)
#define    UxGetResizeHeight(o)       (char *)UxDDGetProp(o, XmNresizeHeight)
#define    UxGetRadioBehavior(o)       (char *)UxDDGetProp(o, XmNradioBehavior)
#define    UxGetRadioAlwaysOne(o)       (char *)UxDDGetProp(o, XmNradioAlwaysOne)
#define    UxGetPopupEnabled(o)       (char *)UxDDGetProp(o, XmNpopupEnabled)
#define    UxGetPacking(o)       (char *)UxDDGetProp(o, XmNpacking)
#define    UxGetOrientation(o)       (char *)UxDDGetProp(o, XmNorientation)
#define    UxGetNumColumns(o)       (short)UxShortGetProp(o, XmNnumColumns)
#define    UxGetMenuPost(o)       (char *)UxDDGetProp(o, XmNmenuPost)
#define    UxGetMenuHistory(o)       (char *)UxDDGetProp(o, XmNmenuHistory)
#define    UxGetMenuHelpWidget(o)       (char *)UxDDGetProp(o, XmNmenuHelpWidget)
#define    UxGetMenuAccelerator(o)       (char *)UxGetProp(o, XmNmenuAccelerator)
#define    UxGetIsHomogeneous(o)       (char *)UxDDGetProp(o, XmNisHomogeneous)
#define    UxGetIsAligned(o)       (char *)UxDDGetProp(o, XmNisAligned)
#define    UxGetEntryVerticalAlignment(o)       (char *)UxDDGetProp(o, XmNentryVerticalAlignment)
#define    UxGetEntryClass(o)       (char *)UxDDGetProp(o, XmNentryClass)
#define    UxGetEntryBorder(o)       (int)UxDDGetProp(o, XmNentryBorder)
#define    UxGetEntryAlignment(o)       (char *)UxDDGetProp(o, XmNentryAlignment)
#define    UxGetAdjustMargin(o)       (char *)UxDDGetProp(o, XmNadjustMargin)
#define    UxGetAdjustLast(o)       (char *)UxDDGetProp(o, XmNadjustLast)
#define    UxGetShowAsDefault(o)       (int)UxDDGetProp(o, XmNshowAsDefault)
#define    UxGetFillOnArm(o)       (char *)UxDDGetProp(o, XmNfillOnArm)
#define    UxGetDefaultButtonShadowThickness(o)       (int)UxDDGetProp(o, XmNdefaultButtonShadowThickness)
#define    UxGetArmPixmap(o)       (char *)UxDDGetProp(o, XmNarmPixmap)
#define    UxGetArmColor(o)       (char *)UxDDGetProp(o, XmNarmColor)
#define    UxGetSkipAdjust(o)       (char *)UxDDGetProp(o, XmNskipAdjust)
#define    UxGetPositionIndex(o)       (short)UxShortGetProp(o, XmNpositionIndex)
#define    UxGetPaneMinimum(o)       (int)UxDDGetProp(o, XmNpaneMinimum)
#define    UxGetPaneMaximum(o)       (int)UxDDGetProp(o, XmNpaneMaximum)
#define    UxGetAllowResize(o)       (char *)UxDDGetProp(o, XmNallowResize)
#define    UxGetSpacing(o)       (int)UxDDGetProp(o, XmNspacing)
#define    UxGetSeparatorOn(o)       (char *)UxDDGetProp(o, XmNseparatorOn)
#define    UxGetSashWidth(o)       (int)UxDDGetProp(o, XmNsashWidth)
#define    UxGetSashShadowThickness(o)       (int)UxDDGetProp(o, XmNsashShadowThickness)
#define    UxGetSashIndent(o)       (int)UxDDGetProp(o, XmNsashIndent)
#define    UxGetSashHeight(o)       (int)UxDDGetProp(o, XmNsashHeight)
#define    UxGetRefigureMode(o)       (char *)UxDDGetProp(o, XmNrefigureMode)
#define    UxGetSymbolPixmap(o)       (char *)UxDDGetProp(o, XmNsymbolPixmap)
#define    UxGetOkLabelString(o)       (char *)UxDDGetProp(o, XmNokLabelString)
#define    UxGetMinimizeButtons(o)       (char *)UxDDGetProp(o, XmNminimizeButtons)
#define    UxGetMessageString(o)       (char *)UxDDGetProp(o, XmNmessageString)
#define    UxGetMessageAlignment(o)       (char *)UxDDGetProp(o, XmNmessageAlignment)
#define    UxGetHelpLabelString(o)       (char *)UxDDGetProp(o, XmNhelpLabelString)
#define    UxGetDefaultButtonType(o)       (char *)UxDDGetProp(o, XmNdefaultButtonType)
#define    UxGetCancelLabelString(o)       (char *)UxDDGetProp(o, XmNcancelLabelString)
#define    UxGetTopShadowPixmap(o)       (char *)UxDDGetProp(o, XmNtopShadowPixmap)
#define    UxGetTopShadowColor(o)       (char *)UxDDGetProp(o, XmNtopShadowColor)
#define    UxGetInitialFocus(o)       (char *)UxDDGetProp(o, XmNinitialFocus)
#define    UxGetHighlightPixmap(o)       (char *)UxDDGetProp(o, XmNhighlightPixmap)
#define    UxGetHighlightColor(o)       (char *)UxDDGetProp(o, XmNhighlightColor)
#define    UxGetForeground(o)       (char *)UxDDGetProp(o, XmNforeground)
#define    UxGetBottomShadowPixmap(o)       (char *)UxDDGetProp(o, XmNbottomShadowPixmap)
#define    UxGetBottomShadowColor(o)       (char *)UxDDGetProp(o, XmNbottomShadowColor)
#define    UxGetAllowShellResize(o)       (char *)UxDDGetProp(o, XmNallowShellResize)
#define    UxGetDefaultFontList(o)       (char *)UxDDGetProp(o, XmNdefaultFontList)
#define    UxGetShowSeparator(o)       (char *)UxDDGetProp(o, XmNshowSeparator)
#define    UxGetMessageWindow(o)       (char *)UxDDGetProp(o, XmNmessageWindow)
#define    UxGetMenuBar(o)       (char *)UxDDGetProp(o, XmNmenuBar)
#define    UxGetMainWindowMarginWidth(o)       (int)UxDDGetProp(o, XmNmainWindowMarginWidth)
#define    UxGetMainWindowMarginHeight(o)       (int)UxDDGetProp(o, XmNmainWindowMarginHeight)
#define    UxGetCommandWindowLocation(o)       (char *)UxDDGetProp(o, XmNcommandWindowLocation)
#define    UxGetCommandWindow(o)       (char *)UxDDGetProp(o, XmNcommandWindow)
#define    UxGetVisibleItemCount(o)       (int)UxGetProp(o, XmNvisibleItemCount)
#define    UxGetTopItemPosition(o)       (int)UxGetProp(o, XmNtopItemPosition)
#define    UxGetSelectionPolicy(o)       (char *)UxDDGetProp(o, XmNselectionPolicy)
#define    UxGetSelectedItemCount(o)       (int)UxGetProp(o, XmNselectedItemCount)
#define    UxGetSelectedItems(o)       (char *)UxDDGetProp(o, XmNselectedItems)
#define    UxGetScrollBarDisplayPolicy(o)       (char *)UxDDGetProp(o, XmNscrollBarDisplayPolicy)
#define    UxGetListSpacing(o)       (int)UxDDGetProp(o, XmNlistSpacing)
#define    UxGetListSizePolicy(o)       (char *)UxDDGetProp(o, XmNlistSizePolicy)
#define    UxGetListMarginWidth(o)       (int)UxDDGetProp(o, XmNlistMarginWidth)
#define    UxGetListMarginHeight(o)       (int)UxDDGetProp(o, XmNlistMarginHeight)
#define    UxGetItemCount(o)       (int)UxGetProp(o, XmNitemCount)
#define    UxGetItems(o)       (char *)UxDDGetProp(o, XmNitems)
#define    UxGetDoubleClickInterval(o)       (int)UxGetProp(o, XmNdoubleClickInterval)
#define    UxGetAutomaticSelection(o)       (char *)UxDDGetProp(o, XmNautomaticSelection)
#define    UxGetStringDirection(o)       (char *)UxDDGetProp(o, XmNstringDirection)
#define    UxGetRecomputeSize(o)       (char *)UxDDGetProp(o, XmNrecomputeSize)
#define    UxGetMnemonicCharSet(o)       (char *)UxGetProp(o, XmNmnemonicCharSet)
#define    UxGetMnemonic(o)       (char *)UxDDGetProp(o, XmNmnemonic)
#define    UxGetMarginTop(o)       (int)UxDDGetProp(o, XmNmarginTop)
#define    UxGetMarginRight(o)       (int)UxDDGetProp(o, XmNmarginRight)
#define    UxGetMarginLeft(o)       (int)UxDDGetProp(o, XmNmarginLeft)
#define    UxGetMarginBottom(o)       (int)UxDDGetProp(o, XmNmarginBottom)
#define    UxGetLabelType(o)       (char *)UxDDGetProp(o, XmNlabelType)
#define    UxGetLabelString(o)       (char *)UxDDGetProp(o, XmNlabelString)
#define    UxGetLabelPixmap(o)       (char *)UxDDGetProp(o, XmNlabelPixmap)
#define    UxGetLabelInsensitivePixmap(o)       (char *)UxDDGetProp(o, XmNlabelInsensitivePixmap)
#define    UxGetFontList(o)       (char *)UxDDGetProp(o, XmNfontList)
#define    UxGetAlignment(o)       (char *)UxDDGetProp(o, XmNalignment)
#define    UxGetAcceleratorText(o)       (char *)UxDDGetProp(o, XmNacceleratorText)
#define    UxGetAccelerator(o)       (char *)UxGetProp(o, XmNaccelerator)
#define    UxGetUserData(o)       (char *)UxDDGetProp(o, XmNuserData)
#define    UxGetUnitType(o)       (char *)UxDDGetProp(o, XmNunitType)
#define    UxGetTraversalOn(o)       (char *)UxDDGetProp(o, XmNtraversalOn)
#define    UxGetShadowThickness(o)       (int)UxDDGetProp(o, XmNshadowThickness)
#define    UxGetNavigationType(o)       (char *)UxDDGetProp(o, XmNnavigationType)
#define    UxGetHighlightThickness(o)       (int)UxDDGetProp(o, XmNhighlightThickness)
#define    UxGetHighlightOnEnter(o)       (char *)UxDDGetProp(o, XmNhighlightOnEnter)
#define    UxGetQualifySearchDataProc(o)       (void (*)())UxGetProp(o, XmNqualifySearchDataProc)
#define    UxGetPattern(o)       (char *)UxDDGetProp(o, XmNpattern)
#define    UxGetNoMatchString(o)       (char *)UxDDGetProp(o, XmNnoMatchString)
#define    UxGetListUpdated(o)       (char *)UxDDGetProp(o, XmNlistUpdated)
#define    UxGetFilterLabelString(o)       (char *)UxDDGetProp(o, XmNfilterLabelString)
#define    UxGetFileTypeMask(o)       (char *)UxDDGetProp(o, XmNfileTypeMask)
#define    UxGetFileSearchProc(o)       (void (*)())UxGetProp(o, XmNfileSearchProc)
#define    UxGetFileListLabelString(o)       (char *)UxDDGetProp(o, XmNfileListLabelString)
#define    UxGetFileListItemCount(o)       (int)UxGetProp(o, XmNfileListItemCount)
#define    UxGetFileListItems(o)       (char *)UxDDGetProp(o, XmNfileListItems)
#define    UxGetDirSpec(o)       (char *)UxDDGetProp(o, XmNdirSpec)
#define    UxGetDirSearchProc(o)       (void (*)())UxGetProp(o, XmNdirSearchProc)
#define    UxGetDirMask(o)       (char *)UxDDGetProp(o, XmNdirMask)
#define    UxGetDirListLabelString(o)       (char *)UxDDGetProp(o, XmNdirListLabelString)
#define    UxGetDirListItemCount(o)       (int)UxGetProp(o, XmNdirListItemCount)
#define    UxGetDirListItems(o)       (char *)UxDDGetProp(o, XmNdirListItems)
#define    UxGetDirectoryValid(o)       (char *)UxDDGetProp(o, XmNdirectoryValid)
#define    UxGetDirectory(o)       (char *)UxDDGetProp(o, XmNdirectory)
#define    UxGetChildVerticalAlignment(o)       (char *)UxDDGetProp(o, XmNchildVerticalAlignment)
#define    UxGetChildHorizontalSpacing(o)       (int)UxDDGetProp(o, XmNchildHorizontalSpacing)
#define    UxGetChildHorizontalAlignment(o)       (char *)UxDDGetProp(o, XmNchildHorizontalAlignment)
#define    UxGetChildType(o)       (char *)UxDDGetProp(o, XmNchildType)
#define    UxGetTopWidget(o)       (char *)UxDDGetProp(o, XmNtopWidget)
#define    UxGetTopPosition(o)       (int)UxGetProp(o, XmNtopPosition)
#define    UxGetTopOffset(o)       (int)UxGetProp(o, XmNtopOffset)
#define    UxGetTopAttachment(o)       (char *)UxDDGetProp(o, XmNtopAttachment)
#define    UxGetRightWidget(o)       (char *)UxDDGetProp(o, XmNrightWidget)
#define    UxGetRightPosition(o)       (int)UxGetProp(o, XmNrightPosition)
#define    UxGetRightOffset(o)       (int)UxGetProp(o, XmNrightOffset)
#define    UxGetRightAttachment(o)       (char *)UxDDGetProp(o, XmNrightAttachment)
#define    UxGetResizable(o)       (char *)UxDDGetProp(o, XmNresizable)
#define    UxGetLeftWidget(o)       (char *)UxDDGetProp(o, XmNleftWidget)
#define    UxGetLeftPosition(o)       (int)UxGetProp(o, XmNleftPosition)
#define    UxGetLeftOffset(o)       (int)UxGetProp(o, XmNleftOffset)
#define    UxGetLeftAttachment(o)       (char *)UxDDGetProp(o, XmNleftAttachment)
#define    UxGetBottomWidget(o)       (char *)UxDDGetProp(o, XmNbottomWidget)
#define    UxGetBottomPosition(o)       (int)UxGetProp(o, XmNbottomPosition)
#define    UxGetBottomOffset(o)       (int)UxGetProp(o, XmNbottomOffset)
#define    UxGetBottomAttachment(o)       (char *)UxDDGetProp(o, XmNbottomAttachment)
#define    UxGetVerticalSpacing(o)       (int)UxDDGetProp(o, XmNverticalSpacing)
#define    UxGetRubberPositioning(o)       (char *)UxDDGetProp(o, XmNrubberPositioning)
#define    UxGetHorizontalSpacing(o)       (int)UxDDGetProp(o, XmNhorizontalSpacing)
#define    UxGetFractionBase(o)       (int)UxGetProp(o, XmNfractionBase)
#define    UxGetPushButtonEnabled(o)       (char *)UxDDGetProp(o, XmNpushButtonEnabled)
#define    UxGetNumChildren(o)       (int)UxGetProp(o, XmNnumChildren)
#define    UxGetInsertPosition(o)       (Cardinal (*)())UxGetProp(o, XmNinsertPosition)
#define    UxGetChildren(o)       (stringTable)UxDDGetProp(o, XmNchildren)
#define    UxGetPromptString(o)       (char *)UxDDGetProp(o, XmNpromptString)
#define    UxGetHistoryVisibleItemCount(o)       (int)UxGetProp(o, XmNhistoryVisibleItemCount)
#define    UxGetHistoryMaxItems(o)       (int)UxGetProp(o, XmNhistoryMaxItems)
#define    UxGetHistoryItemCount(o)       (int)UxGetProp(o, XmNhistoryItemCount)
#define    UxGetHistoryItems(o)       (char *)UxDDGetProp(o, XmNhistoryItems)
#define    UxGetCommand(o)       (char *)UxDDGetProp(o, XmNcommand)
#define    UxGetSubMenuId(o)       (char *)UxDDGetProp(o, XmNsubMenuId)
#define    UxGetMappingDelay(o)       (int)UxGetProp(o, XmNmappingDelay)
#define    UxGetCascadePixmap(o)       (char *)UxDDGetProp(o, XmNcascadePixmap)
#define    UxGetTextFontList(o)       (char *)UxDDGetProp(o, XmNtextFontList)
#define    UxGetShadowType(o)       (char *)UxDDGetProp(o, XmNshadowType)
#define    UxGetResizePolicy(o)       (char *)UxDDGetProp(o, XmNresizePolicy)
#define    UxGetNoResize(o)       (char *)UxDDGetProp(o, XmNnoResize)
#define    UxGetMarginWidth(o)       (int)UxDDGetProp(o, XmNmarginWidth)
#define    UxGetMarginHeight(o)       (int)UxDDGetProp(o, XmNmarginHeight)
#define    UxGetLabelFontList(o)       (char *)UxDDGetProp(o, XmNlabelFontList)
#define    UxGetDialogTitle(o)       (char *)UxDDGetProp(o, XmNdialogTitle)
#define    UxGetDialogStyle(o)       (char *)UxDDGetProp(o, XmNdialogStyle)
#define    UxGetDefaultPosition(o)       (char *)UxDDGetProp(o, XmNdefaultPosition)
#define    UxGetDefaultButton(o)       (char *)UxDDGetProp(o, XmNdefaultButton)
#define    UxGetCancelButton(o)       (char *)UxDDGetProp(o, XmNcancelButton)
#define    UxGetButtonFontList(o)       (char *)UxDDGetProp(o, XmNbuttonFontList)
#define    UxGetAutoUnmanage(o)       (char *)UxDDGetProp(o, XmNautoUnmanage)
#define    UxGetAllowOverlap(o)       (char *)UxDDGetProp(o, XmNallowOverlap)
#define    UxGetMultiClick(o)       (char *)UxDDGetProp(o, XmNmultiClick)
#define    UxGetArrowDirection(o)       (char *)UxDDGetProp(o, XmNarrowDirection)
#define    UxGetArgv(o)       (char**)UxGetProp(o, XmNargv)
#define    UxGetArgc(o)       (int)UxGetProp(o, XmNargc)
#define    UxGetY(o)       (int)UxDDGetProp(o, XmNy)
#define    UxGetX(o)       (int)UxDDGetProp(o, XmNx)
#define    UxGetWidth(o)       (int)UxDDGetProp(o, XmNwidth)
#define    UxGetSensitive(o)       (char *)UxDDGetProp(o, XmNsensitive)
#define    UxGetHeight(o)       (int)UxDDGetProp(o, XmNheight)
#define    UxGetBorderWidth(o)       (int)UxDDGetProp(o, XmNborderWidth)
#define    UxGetAncestorSensitive(o)       (char *)UxDDGetProp(o, XmNancestorSensitive)
#define    UxGetMappedWhenManaged(o)       (char *)UxDDGetProp(o, XmNmappedWhenManaged)
#define    UxGetColormap(o)       (long)UxDDGetProp(o, XmNcolormap)
#define    UxGetBorderPixmap(o)       (char *)UxDDGetProp(o, XmNborderPixmap)
#define    UxGetBorderColor(o)       (char *)UxDDGetProp(o, XmNborderColor)
#define    UxGetBackgroundPixmap(o)       (char *)UxDDGetProp(o, XmNbackgroundPixmap)
#define    UxGetBackground(o)       (char *)UxDDGetProp(o, XmNbackground)
#endif /* _UXGETSMF_INCLUDED */
