C===========================================================================
C Copyright (C) 1995 European Southern Observatory (ESO)
C
C This program is free software; you can redistribute it and/or
C modify it under the terms of the GNU General Public License as
C published by the Free Software Foundation; either version 2 of
C the License, or (at your option) any later version.
C
C This program is distributed in the hope that it will be useful,
C but WITHOUT ANY WARRANTY; without even the implied warranty of
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C GNU General Public License for more details.
C
C You should have received a copy of the GNU General Public
C License along with this program; if not, write to the Free
C Software Foundation, Inc., 675 Massachusetss Ave, Cambridge,
C MA 02139, USA.
C
C Correspondence concerning ESO-MIDAS should be addressed as follows:
C       Internet e-mail: midas@eso.org
C       Postal address: European Southern Observatory
C                       Data Management Division
C                       Karl-Schwarzschild-Strasse 2
C                       D 85748 Garching bei Muenchen
C                       GERMANY
C===========================================================================

      SUBROUTINE IICINC(DSPLAY,CHAN,CURSNO,SHAPE,COLOUR,XC,YC,IDST)
C
      IMPLICIT NONE
C
      INTEGER     DSPLAY,CHAN
      INTEGER     CURSNO,SHAPE,COLOUR,XC,YC,IDST
C
      CALL IIC1(DSPLAY,CHAN,CURSNO,SHAPE)
      CALL IIC2(COLOUR,XC,YC,IDST)
C
      RETURN
      END

      SUBROUTINE IICRCP(DSPLAY,CHAN,CURSNO,XC,YC,MEMO,IDST)
C
      IMPLICIT NONE
C
      INTEGER     DSPLAY,CHAN
      INTEGER     CURSNO,XC,YC,MEMO,IDST
C
      CALL IIC1(DSPLAY,CHAN,CURSNO,XC)
      CALL IIC3(YC,MEMO,IDST)
C
      RETURN
      END

