C @(#)iic.for	19.1 (ES0-DMD) 02/25/03 13:55:00
C===========================================================================
C Copyright (C) 1995 European Southern Observatory (ESO)
C
C This program is free software; you can redistribute it and/or 
C modify it under the terms of the GNU General Public License as 
C published by the Free Software Foundation; either version 2 of 
C the License, or (at your option) any later version.
C
C This program is distributed in the hope that it will be useful,
C but WITHOUT ANY WARRANTY; without even the implied warranty of
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C GNU General Public License for more details.
C
C You should have received a copy of the GNU General Public 
C License along with this program; if not, write to the Free 
C Software Foundation, Inc., 675 Massachusetss Ave, Cambridge, 
C MA 02139, USA.
C
C Corresponding concerning ESO-MIDAS should be addressed as follows:
C	Internet e-mail: midas@eso.org
C	Postal address: European Southern Observatory
C			Data Management Division 
C			Karl-Schwarzschild-Strasse 2
C			D 85748 Garching bei Muenchen 
C			GERMANY
C===========================================================================
C
      SUBROUTINE IICINC(DSPLAY,CHAN,CURSNO,SHAPE,COLOUR,XC,YC,IDST)
C
      IMPLICIT NONE
C	
      INTEGER   DSPLAY,CHAN,CURSNO,SHAPE,COLOUR,XC,YC,IDST
C 
      INCLUDE   'MID_INCLUDE:IDIDEV.INC'
C 
      IF (IDINUM.EQ.0) THEN
         CALL GD8029(DSPLAY,CHAN,CURSNO,SHAPE,COLOUR,XC,YC,IDST)
      ELSE IF (IDINUM.EQ.10) THEN
         CALL MXW029(DSPLAY,CHAN,CURSNO,SHAPE,COLOUR,XC,YC,IDST)
      ELSE
         IDST = 0
      ENDIF
C	
      RETURN
      END
C 
C 
C -----------------------------------------------------------------
C 
C 
      SUBROUTINE IICSCV(DSPLAY,CURSNO,VIS,IDST)
C
      IMPLICIT NONE
C	
      INTEGER   DSPLAY,CURSNO,VIS,IDST
C 
      INCLUDE   'MID_INCLUDE:IDIDEV.INC'
C 
      IF (IDINUM.EQ.0) THEN
         CALL GD8030(DSPLAY,CURSNO,VIS,IDST)
      ELSE IF (IDINUM.EQ.10) THEN
         CALL MXW030(DSPLAY,CURSNO,VIS,IDST)
      ELSE
         IDST = 0
      ENDIF
C	
      RETURN
      END

      SUBROUTINE IICWCP(DSPLAY,CHAN,CURSNO,XC,YC,IDST)
C
      IMPLICIT NONE
C	
      INTEGER   DSPLAY,CHAN,CURSNO,XC,YC,IDST
C 
      INCLUDE   'MID_INCLUDE:IDIDEV.INC'
C 
      IF (IDINUM.EQ.0) THEN
         CALL GD8032(DSPLAY,CHAN,CURSNO,XC,YC,IDST)
      ELSE IF (IDINUM.EQ.10) THEN
         CALL MXW032(DSPLAY,CHAN,CURSNO,XC,YC,IDST)
      ELSE
         IDST = 0
      ENDIF
C	
      RETURN
      END
C 
C 
C -----------------------------------------------------------------
C 
C 
      SUBROUTINE IICRCP(DSPLAY,CHAN,CURSNO,XC,YC,MEMO,IDST)
C
      IMPLICIT NONE
C	
      INTEGER   DSPLAY,CHAN,CURSNO,XC,YC,MEMO,IDST
C 
      INCLUDE   'MID_INCLUDE:IDIDEV.INC'
C 
      IF (IDINUM.EQ.0) THEN
         CALL GD8031(DSPLAY,CHAN,CURSNO,XC,YC,MEMO,IDST)
      ELSE IF (IDINUM.EQ.10) THEN
         CALL MXW031(DSPLAY,CHAN,CURSNO,XC,YC,MEMO,IDST)
      ELSE
         IDST = 0
      ENDIF
C	
      RETURN
      END
